/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_PhilipsXA.h                      *
*                 Philips-XA Specific Code                 *
*                    LARGE MEMORY MODEL                    *
***********************************************************/
#ifndef _OS_PhilipsXA_
 #define _OS_PhilipsXA_

 #include <REGXAS3.h>         /* from RIDE                */

/*
************************************************************
*               MACROS (compiler spezific)
************************************************************
*/
 #define  OS_CRITICAL_SR       0                             // do not save the CPU-register during INT-disable

 #define  OS_ENTER_CRITICAL()  EA = 0
 #define  OS_EXIT_CRITICAL()   EA = 1
 #define  OS_TASK_SW()         asm{ 0xD6, 0x3F }    
            /* Trap #15, (Interrupt 31) invokes OSCtxSw() */
/*
************************************************************
*                    ENDIAN CONVENTION
************************************************************
*/
 #define  LITTLE_ENDIAN   3412
 #define  BIG_ENDIAN      1234

 #define  BYTE_ORDER  LITTLE_ENDIAN   /* endianess of CPU */

//------------ for little-endian protocols -----------------
 #if BYTE_ORDER == LITTLE_ENDIAN                           // for example: USB 
  #define L_htonl(n)       (n)
  #define L_htons(n)       (n)
 #else
  #define L_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define L_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  L_ntohl          L_htonl
 #define  L_ntohs          L_htons
//-------------- for big-endian protocols ------------------
 #if BYTE_ORDER == BIG_ENDIAN                              // for example: 802.3 , TCP/IP
  #define B_htonl(n)       (n)
  #define B_htons(n)       (n)
 #else
  #define B_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define B_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  B_ntohl          B_htonl
 #define  B_ntohs          B_htons
/*
************************************************************
*                 POINTER AND REENTRANCE
************************************************************
*/
 #define  OS_EXT  extern   /* define external declaration */
 #define  OS_FAR              /* not needed               */
 #define  OS_HUGE             /* define OS_HUGE           */
 #define  OS_CONST const      /* define as const          */
 #define  OS_ROMDATA code     /* area locations in ROM    */
 #define  KC51_REENTRANT                    /* not needed */
/*
************************************************************
*                      DATA TYPES
************************************************************
*/
 typedef unsigned char  BOOLEAN;
 typedef unsigned char  U08;
 typedef signed   char  S08;
 typedef unsigned short U16;
 typedef signed   short S16;
 typedef unsigned long  U32;
 typedef signed   long  S32;
 typedef float          FP;
 typedef U16            OS_STK_TYPE;

/*
************************************************************
*                       CONSTANTS
************************************************************
*/
 #ifndef FALSE
  #define FALSE  0
 #endif
 #ifndef TRUE
  #define TRUE   1
 #endif
 #ifndef NULL
  #define NULL   ((void *)0)
 #endif

/*
************************************************************
*                  COMPILING DEFINITIONS
************************************************************
*/
#ifndef OS_PACKED
 #define OS_PACKED
#endif
#ifndef OS_PACKED_ATTR
 #define OS_PACKED_ATTR
#endif
#ifndef OS_ALIGN_ATTR
 #define OS_ALIGN_ATTR
#endif

/************************** END ***************************/
#endif
