/*************************************************************
*
* Description:
*	pC/OS-51 - Example program (Keil-C51 - large model)
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   512

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08         pool[2048];

OS_MUX Mutex1;
OS_EVG Events1;

void  Task1(void *Tdata) KC51_REENTRANT;
void  Task2(void *Tdata) KC51_REENTRANT;
void  Task3(void *Tdata) KC51_REENTRANT;


void main()
{
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), 2048);

    OS_TaskCreate(Task1, NULL, TASK_STK_SIZE, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, TASK_STK_SIZE, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, TASK_STK_SIZE, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
    OS_Start();
}

void AppTickInit(void) KC51_REENTRANT
{
    TH0  =    0;
    TL0  =    0;
    TMOD = 0x01;
    ET0  =    1;
    TR0  =    1;
}


void Task1(void *Tdata) KC51_REENTRANT
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    Tdata = Tdata;
    i=0;
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        while(i<2500) i++;                              // no Task-switch to MUTEX
        i=0;
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(100);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(100);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}


void Task2(void *Tdata) KC51_REENTRANT
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // suspending
        if(!returnOk) {
            while(i<2500) i++;                          // no Taskswitch to Task1 or Task3
            i=0;
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(200);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

void Task3(void *Tdata) KC51_REENTRANT
{
    U16 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        OS_TimeDly(10);                                 // for Task2
        i++;
    }
}


