;****************************************************************
;*                       pC/OS 1.68a                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                    OS_ATMega128_a.asm                        *
;*                    AVR Specific Code                         *
;*                      using WinAVR                            *
;****************************************************************
#include <avr/io.h>               // for "SIG_OVERFLOW0" interrupt

    .text
    .section    .text

    .global     OSStartHighRdy
    .global     OSCtxSw
    .global     OSIntCtxSw
    .global     OSTickISR

    .extern     OSTCBHighRdy
    .extern     OSTCBCur
    .extern     OSIntNesting
    .extern     OSTickISR2
    .extern     OSIntExit
	
#define _SREG    0x3F
#define _SPH     0x3E
#define _SPL     0x3D
#define _RAMPZ   0x3B
#define _TCNT0   0x52

;****************************************************************
; The following macro is the assembly code to save the current 
; task context
.macro  save_context  
        PUSH    R0
        PUSH    R1
        PUSH    R2
        PUSH    R3
        PUSH    R4
        PUSH    R5
        PUSH    R6
        PUSH    R7
        PUSH    R8
        PUSH    R9
        PUSH    R10
        PUSH    R11
        PUSH    R12
        PUSH    R13
        PUSH    R14
        PUSH    R15
        PUSH    R16
        PUSH    R17
        PUSH    R18
        PUSH    R19
        PUSH    R20
        PUSH    R21
        PUSH    R22
        PUSH    R23
        PUSH    R24
        PUSH    R25
        PUSH    R26
        PUSH    R27
        PUSH    R28
        PUSH    R29
        PUSH    R30
        PUSH    R31
        IN      R16,_RAMPZ
        PUSH    R16
        IN      R16,_SREG
        PUSH    R16
.endm

;****************************************************************
; The following macro is the assembly code to restore the current 
; task context
.macro  restore_context
        POP     R16
        OUT     _SREG,R16
        POP     R16
        OUT     _RAMPZ,R16
        POP     R31
        POP     R30
        POP     R29
        POP     R28
        POP     R27
        POP     R26
        POP     R25
        POP     R24
        POP     R23
        POP     R22
        POP     R21
        POP     R20
        POP     R19
        POP     R18
        POP     R17
        POP     R16
        POP     R15
        POP     R14
        POP     R13
        POP     R12
        POP     R11
        POP     R10
        POP     R9
        POP     R8
        POP     R7
        POP     R6
        POP     R5
        POP     R4
        POP     R3
        POP     R2
        POP     R1
        POP     R0
.endm

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
OSStartHighRdy:
        LDS     R30,OSTCBHighRdy            ; Let Z point to TCB of highest priority task
        LDS     R31,OSTCBHighRdy+1          ; ready to run
        STS     OSTCBCur,R30                ; save as current
        STS     OSTCBCur+1,R31              ;
        LD      R28,Z+                      ; Load Y (R29:R28) pointer ...
        LD      R29,Z+                      ;
        OUT     _SPL,R28                    ; ... to SP
        OUT     _SPH,R29                    ; 
        restore_context                     ; restore stack-ptr, status, registers
        RET                                 ; Start task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
OSCtxSw:
        save_context                        ; save registers, status, stack-ptr
        LDS     R30,OSTCBCur                ; Save stack ptr in TCB
        LDS     R31,OSTCBCur+1              ;
        IN      R28,_SPL                    ; get SP to Y pointer ...
        IN      R29,_SPH                    ;
        ST      Z+,R28                      ; ... save Y (R29:R28) pointer
        ST      Z+,R29                      ;
        LDS     R30,OSTCBHighRdy            ; Let Z point to TCB of highest priority task
        LDS     R31,OSTCBHighRdy+1          ; ready to run
        STS     OSTCBCur,R30                ; OSTCBCur = OSTCBHighRdy
        STS     OSTCBCur+1,R31              ;
        LD      R28,Z+                      ; Restore Y pointer ...
        LD      R29,Z+                      ;
        OUT     _SPL,R28                    ; ... to SP
        OUT     _SPH,R29                    ; 
        restore_context                     ; restore stack-ptr, status, registers
        RET                                 ; Return to new task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
OSIntCtxSw:
        LDS     R30,OSTCBHighRdy            ; Z = OSTCBHighRdy->OSTCBStkPtr
        LDS     R31,OSTCBHighRdy+1          ;
        STS     OSTCBCur,R30                ; OSTCBCur = OSTCBHighRdy
        STS     OSTCBCur+1,R31              ;
        LD      R28,Z+                      ; Restore Y pointer ...
        LD      R29,Z+                      ;
        OUT     _SPL,R28                    ; ... to SP
        OUT     _SPH,R29                    ; 
        restore_context                     ; restore stack-ptr, status, registers
        RETI                                ; Return to new task

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
.global SIG_OVERFLOW0
SIG_OVERFLOW0:
OSTickISR:
        save_context                        ; save registers, status, stack-ptr
        LDS     R30,OSIntNesting            ; register the ISR-level  
        INC     R30                         ; (do not call OSIntEnter ! - since the..
        STS     OSIntNesting,R30            ;  ..INTs can be reenabled inside an ISR)
        CPI     R30,1                       ; if OSIntNesting == 1 ..
        BRNE    OSTickISR_1                 ;
        LDS     R30,OSTCBCur                ; .. save stack ptr in TCB
        LDS     R31,OSTCBCur+1              ;
        IN      R28,_SPL                    ; get SP to Y pointer ...
        IN      R29,_SPH                    ;
        ST      Z+,R28                      ; ... save Y (R29:R28) pointer
        ST      Z+,R29                      ;
OSTickISR_1:
        CALL    OSTickISR2                  ; CALL OS-TIMER FUNCTION
        CALL    OSIntExit                   ; perform a context-switch if nesesary
        restore_context                     ; restore stack-ptr, status, registers
        RETI                                ; Return to task


