/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                    OS_ATMega32_c.c                       *
*                   AVR Specific Code                      *
*                      IAR version                         *
***********************************************************/
#include "../../../inc/pC_OS.h"
#include "../../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk, *hstk;
    U16  tmp;

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
    hstk     = stk - stksize + OS_HW_STACKSIZE;
    tmp      = (U16)task;
    *hstk--  = (U08)(tmp);                                /* task start address                   */
    *hstk--  = (U08)(tmp >> 8);
    *--stk   = (U08)0x00;                                 /* R0    = 0x00                         */
    *--stk   = (U08)0x01;                                 /* R1    = 0x01                         */
    *--stk   = (U08)0x02;                                 /* R2    = 0x02                         */
    *--stk   = (U08)0x03;                                 /* R3    = 0x03                         */
    *--stk   = (U08)0x04;                                 /* R4    = 0x04                         */
    *--stk   = (U08)0x05;                                 /* R5    = 0x05                         */
    *--stk   = (U08)0x06;                                 /* R6    = 0x06                         */
    *--stk   = (U08)0x07;                                 /* R7    = 0x07                         */
    *--stk   = (U08)0x08;                                 /* R8    = 0x08                         */
    *--stk   = (U08)0x09;                                 /* R9    = 0x09                         */
    *--stk   = (U08)0x10;                                 /* R10   = 0x10                         */
    *--stk   = (U08)0x11;                                 /* R11   = 0x11                         */
    *--stk   = (U08)0x12;                                 /* R12   = 0x12                         */
    *--stk   = (U08)0x13;                                 /* R13   = 0x13                         */
    *--stk   = (U08)0x14;                                 /* R14   = 0x14                         */
    *--stk   = (U08)0x15;                                 /* R15   = 0x15                         */
    tmp      = (U16)Tdata;
    *--stk   = (U08)(tmp);                                /* data passed in R17:R16               */
    *--stk   = (U08)(tmp >> 8);
    *--stk   = (U08)0x18;                                 /* R18   = 0x18                         */
    *--stk   = (U08)0x19;                                 /* R19   = 0x19                         */
    *--stk   = (U08)0x20;                                 /* R20   = 0x20                         */
    *--stk   = (U08)0x21;                                 /* R21   = 0x21                         */
    *--stk   = (U08)0x22;                                 /* R22   = 0x22                         */
    *--stk   = (U08)0x23;                                 /* R23   = 0x23                         */
    *--stk   = (U08)0x24;                                 /* R24   = 0x24                         */
    *--stk   = (U08)0x25;                                 /* R25   = 0x25                         */
    *--stk   = (U08)0x26;                                 /* R26   = 0x26                         */
    *--stk   = (U08)0x27;                                 /* R27   = 0x27                         */
                                        /* R28     R29:R28 is the software stack which gets ...     */
                                        /* R29     ... stored in the task's OS_TCB.                 */
    *--stk   = (U08)0x30;                                 /* R30   = 0x30                         */
    *--stk   = (U08)0x31;                                 /* R31   = 0x31                         */
    *--stk   = (U08)0x80;                                 /* SREG  = Interrupts enabled           */
    tmp      = (U16)(hstk);
    *--stk   = (U08)(tmp >> 8);                           /* SPH                                  */
    *--stk   = (U08)(tmp);                                /* SPL                                  */
#if OS_STK_CHECK_EN
 #error "stack-checking not supported on this HW platform !"
#endif
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
