/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_AVR32_c.c                        *
*                  AVR32 Specific Code                     *
*                    using IAR / GCC                       *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
    *--stk   = (U32)0x08080808L;                          /* R8                                   */
    *--stk   = (U32)0x09090909L;                          /* R9                                   */
    *--stk   = (U32)0x10101010L;                          /* R10                                  */
    *--stk   = (U32)0x11111111L;                          /* R11                                  */
    *--stk   = (U32)Tdata;                                /* R12 : argument                       */
#ifdef __GNUC__
    *--stk   = (U32)0x80000000L;                          /* R14 (LR) reset vector for GNU-C      */
#else
    *--stk   = (U32)0x14141414L;                          /* R14 (LR)                             */
#endif
    *--stk   = (U32)(task);                               /* R15 (PC)                             */
    *--stk   = (U32)0x00400000L;                          /* SR  : supervisor mode                */
    *--stk   = (U32)0xFF0000FFL;                          /* R0                                   */
    *--stk   = (U32)0x01010101L;                          /* R1                                   */
    *--stk   = (U32)0x02020202L;                          /* R2                                   */
    *--stk   = (U32)0x03030303L;                          /* R3                                   */
    *--stk   = (U32)0x04040404L;                          /* R4                                   */
    *--stk   = (U32)0x05050505L;                          /* R5                                   */
    *--stk   = (U32)0x06060606L;                          /* R6                                   */
#ifdef __GNUC__
    *--stk   = (U32)0x00000000L;                          /* R7    for AVR32Studio                */
#else
    *--stk   = (U32)0x07070707L;                          /* R7                                   */
#endif
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
