/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_FE310_c.c                         *
*               FE310 RISC-V Specific Code                 *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;

    // ---- HW specific ----
    register U32 _gp asm("%x3");
    register U32 _mi = read_csr(mstatus);
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
    *--stk   = (U32)(task);                               /* mepc / x1'     : return address      */
    *--stk   = (U32)(_mi | 0x80);                         /* MPIE from MIE  : interrupts enabled  */
    *--stk   = (U32)0x31313131L;                          /* x31  / t6      : temp                */
    *--stk   = (U32)0x30303030L;                          /* x30  / t5      : temp                */
    *--stk   = (U32)0x29292929L;                          /* x29  / t4      : temp                */
    *--stk   = (U32)0x28282828L;                          /* x28  / t3      : temp                */
    *--stk   = (U32)0x27272727L;                          /* x27  / s11     : saved register      */
    *--stk   = (U32)0x26262626L;                          /* x26  / s10     : saved register      */
    *--stk   = (U32)0x25252525L;                          /* x25  / s9      : saved register      */
    *--stk   = (U32)0x24242424L;                          /* x24  / s8      : saved register      */
    *--stk   = (U32)0x23232323L;                          /* x23  / s7      : saved register      */
    *--stk   = (U32)0x22222222L;                          /* x22  / s6      : saved register      */
    *--stk   = (U32)0x21212121L;                          /* x21  / s5      : saved register      */
    *--stk   = (U32)0x20202020L;                          /* x20  / s4      : saved register      */
    *--stk   = (U32)0x19191919L;                          /* x19  / s3      : saved register      */
    *--stk   = (U32)0x18181818L;                          /* x18  / s2      : saved register      */
    *--stk   = (U32)0x17171717L;                          /* x17  / a7      : argument            */
    *--stk   = (U32)0x16161616L;                          /* x16  / a6      : argument            */
    *--stk   = (U32)0x15151515L;                          /* x15  / a5      : argument            */
    *--stk   = (U32)0x14141414L;                          /* x14  / a4      : argument            */
    *--stk   = (U32)0x13131313L;                          /* x13  / a3      : argument            */
    *--stk   = (U32)0x12121212L;                          /* x12  / a2      : argument            */
    *--stk   = (U32)0x11111111L;                          /* x11  / a1      : argument / return   */
    *--stk   = (U32)Tdata;                                /* x10  / a0      : argument / return   */
    *--stk   = (U32)0x09090909L;                          /* x9   / s1      : saved register      */
    *--stk   = (U32)0x08080808L;                          /* x8   / s0 / fp : saved register / frame pointer  */
    *--stk   = (U32)0x07070707L;                          /* x7   / t2      : temp                */
    *--stk   = (U32)0x06060606L;                          /* x6   / t1      : temp                */
    *--stk   = (U32)0x05050505L;                          /* x5   / t0      : temp                */
    *--stk   = (U32)0x04040404L;                          /* x4   / tp      : thread pointer      */
    *--stk   = _gp;                                       /* x3   / gp      : global pointer      */
    *--stk   = (U32)0x02020202L;                          /* x2   / sp      : stack pointer       */
    *--stk   = (U32)(task);                               /* x1   / ra      : return address      */
    *--stk   = (U32)0x00000000L;                          /* x0   / zero    : hard-wired zero     */
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
