/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_LPC236x_c.c                       *
*                   ARM Specific Code                      *
*                       (ARM/THUMB)                        *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

#define  ARM_MODE_ARM     0x00000000
#define  ARM_MODE_THUMB   0x00000020

#ifdef __IAR_SYSTEMS_ICC__
 #if __CPU_MODE__ == 1
  #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
 #else
  #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
 #endif
#else
 #ifdef __KEIL__
  #ifdef __THUMB__
   #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
  #else
   #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
  #endif
 #else
  #ifdef __IMAGECRAFT__
   #ifdef THUMB
    #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
   #else
    #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
   #endif
  #else
   #ifdef __GNUC__
    #ifdef __thumb__              // or #ifndef __arm__
     #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
    #else
     #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
    #endif
   #else
    #ifdef __CC_ARM
     #ifdef __APCS_INTERWORK
      #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
     #else
      #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
     #endif
    #endif
   #endif
  #endif
 #endif
#endif

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
    *--stk   = (U32)(task);                               /* task start address                   */
    *--stk   = (U32)0x14141414L;                          /* R14 (LR)                             */
    *--stk   = (U32)0x12121212L;                          /* R12                                  */
    *--stk   = (U32)0x11111111L;                          /* R11                                  */
    *--stk   = (U32)0x10101010L;                          /* R10                                  */
    *--stk   = (U32)0x09090909L;                          /* R9                                   */
    *--stk   = (U32)0x08080808L;                          /* R8                                   */
    *--stk   = (U32)0x07070707L;                          /* R7                                   */
    *--stk   = (U32)0x06060606L;                          /* R6                                   */
    *--stk   = (U32)0x05050505L;                          /* R5                                   */
    *--stk   = (U32)0x04040404L;                          /* R4                                   */
    *--stk   = (U32)0x03030303L;                          /* R3                                   */
    *--stk   = (U32)0x02020202L;                          /* R2                                   */
    *--stk   = (U32)0x01010101L;                          /* R1                                   */
    *--stk   = (U32)Tdata;                                /* R0 : argument                        */
    *--stk   = (U32)ARM_SVC_MODE;                         /* CPSR  (Enable IRQ and FIQ)           */
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
