/*************************************************************
*
* Description:
*	LPC43xx - Example program using Keil
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

/**********************************
* user LEDs                       *
* P6_12 - LED1 green              *
* P6_11 - LED2 yellow             *
* P6_10 - LED3 red                *
* P6_9  - LED4 orange             *
**********************************/

#define LED_M0_PORT
#define LED_M0_PIN

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void SysTick_init(void)
{
    SystemCoreClockUpdate();
    SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC);
}

void SysTick_update(void)
{
}

#define CCU_CLK_CFG_RUN   (1 << 0)
#define CCU_CLK_CFG_AUTO  (1 << 1)
#define CCU_CLK_STAT_RUN  (1 << 0)
U08 LED;
void LED_init(void)
{
    LPC_CCU1->CLK_M4_GPIO_CFG  = CCU_CLK_CFG_AUTO | CCU_CLK_CFG_RUN;
    while (!(LPC_CCU1->CLK_M4_GPIO_STAT & CCU_CLK_STAT_RUN));

    LPC_SCU->SFSP6_9       = 0x00000028;
    LPC_GPIO_PORT->DIR[3] |= (1 << 5);
    LPC_GPIO_PORT->SET[3]  = (1 << 5);
	
    LPC_SCU->SFSP6_10      = 0x00000028;
    LPC_GPIO_PORT->DIR[3] |= (1 << 6);
    LPC_GPIO_PORT->SET[3]  = (1 << 6);
	
    LPC_SCU->SFSP6_11      = 0x00000028;
    LPC_GPIO_PORT->DIR[3] |= (1 << 7);
    LPC_GPIO_PORT->SET[3]  = (1 << 7);

    LPC_SCU->SFSP6_12      = 0x00000028;
    LPC_GPIO_PORT->DIR[2] |= (1 << 8);
    LPC_GPIO_PORT->SET[2]  = (1 << 8);
    LED = 0;
}

void LED_toggle(void)
{
    if(LED) LPC_GPIO_PORT->CLR[2]  = (1 << 8);
    else    LPC_GPIO_PORT->SET[2]  = (1 << 8);
    LED = (LED)? 0:1;
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    LPC_CREG->M0APPMEMMAP = 0x1B000000;
    LPC_RGU->RESET_CTRL1  = 0;          // start CM0 core
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical Cortex-M4 pC/OS ISR
{
    OS_IntEnter();

    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
    OS_IntExit();
}

void AppTickInit(void)
{                                                          // configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;
 
    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timer-triggered signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
