/*
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ----------------------------------------------------------------------
 *
 * $Date:        15. October 2016
 * $Revision:    1.1.0
 *
 * Project:      TrustZone for ARMv8-M
 * Title:        Code template for secure main function
 *
 *---------------------------------------------------------------------------*/
 
/* Use CMSE intrinsics */
#include <arm_cmse.h>

#include <stdio.h>
#include "M2351.h"                        /* Device header */
 
/* TZ_START_NS: Start address of non-secure application */
#ifndef TZ_START_NS
 #define TZ_START_NS (0x10040000U)
#endif
 
/* typedef for non-secure callback functions */
typedef void (*funcptr_void) (void) __attribute__((cmse_nonsecure_call));
 
/*
	The Secure Application:
	* Sets the non-secure main stack (MSP_NS) 
	* Gets the non-secure reset handler
	* Jumps to the non-secure software application
*/



/* typedef for NonSecure callback functions */
typedef __NONSECURE_CALL int32_t (*NonSecure_funcptr)(uint32_t);

/* Secure main() */
int main(void)
{
    funcptr_void NonSecure_ResetHandler;
 
    SYS_UnlockReg();

    /* Add user setup code for secure part here*/
   
        /* (1) set clock up to 64MHz */
        CLK->PLLCTL = CLK_PLLCTL_128MHz_HIRC;
        while((CLK->STATUS & CLK_STATUS_PLLSTB_Msk) == 0);
        CLK->CLKDIV0 = (CLK->CLKDIV0 & (~CLK_CLKDIV0_HCLKDIV_Msk)) | 1;
        CLK->CLKSEL0 = (CLK->CLKSEL0 & (~CLK_CLKSEL0_HCLKSEL_Msk)) | CLK_CLKSEL0_HCLKSEL_PLL;
        CLK->PCLKDIV = (CLK_PCLKDIV_APB0DIV_HCLK_DIV2 | CLK_PCLKDIV_APB1DIV_HCLK_DIV2); 
        PllClock        = 128000000;           // PLL
        SystemCoreClock = 64000000 / 1;        // HCLK
        CyclesPerUs     = 64000000 / 1000000;  // For SYS_SysTickDelay()

        /* (2) Allocate PA11 (LED pin) to Non Secure world */
        // done in "partition_M2351.h"

        /* (3) Enable SPI0 peripheral clock (32MHz) & pins (to none-secure in "partition_M2351.h") */
        SYS->GPF_MFPL = ((SYS->GPF_MFPL & ~(SYS_GPF_MFPL_PF6MFP_Msk)) | SYS_GPF_MFPL_PF6MFP_SPI0_MOSI);
        SYS->GPF_MFPL = ((SYS->GPF_MFPL & ~(SYS_GPF_MFPL_PF7MFP_Msk)) | SYS_GPF_MFPL_PF7MFP_SPI0_MISO);
        SYS->GPF_MFPH = ((SYS->GPF_MFPH & ~(SYS_GPF_MFPH_PF8MFP_Msk)) | SYS_GPF_MFPH_PF8MFP_SPI0_CLK);
        SYS->GPF_MFPH = ((SYS->GPF_MFPH & ~(SYS_GPF_MFPH_PF9MFP_Msk)) | SYS_GPF_MFPH_PF9MFP_SPI0_SS);
        CLK_SetModuleClock(SPI0_MODULE, CLK_CLKSEL2_SPI0SEL_PCLK1, MODULE_NoMsk);
        CLK_EnableModuleClock(SPI0_MODULE);

        /* (4) Enable I2C1 peripheral clock & pins (to none-secure in "partition_M2351.h") */
        SYS->GPG_MFPL &= ~(SYS_GPG_MFPL_PG2MFP_Msk | SYS_GPG_MFPL_PG3MFP_Msk);
        SYS->GPG_MFPL |= (SYS_GPG_MFPL_PG2MFP_I2C1_SCL | SYS_GPG_MFPL_PG3MFP_I2C1_SDA); 
        CLK_EnableModuleClock(I2C1_MODULE);
 
    SCB_NS->VTOR = (uint32_t)(TZ_START_NS);
   
    /* Set non-secure main stack (MSP_NS) */
    __TZ_set_MSP_NS(*((uint32_t *)(TZ_START_NS)));
 
    /* Get non-secure reset handler */
    NonSecure_ResetHandler = (funcptr_void)(*((uint32_t *)((TZ_START_NS) + 4U)));
 
    /* Start non-secure state software application */
    NonSecure_ResetHandler();

    /* Non-secure software does not return, this code is not executed */
    while (1) {
        __NOP();
    }
}
