/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_PIC24_c.c                         *
*       PIC24FJ Harvard Architecture Specific Code         *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk, x, h;

    // ---- HW specific ----
    x = 0; h = 0;
    if (CORCONbits.IPL3) x |= 0x0080;
    stk      = (OS_STK_TYPE *)*pstk;
    *pstk    = (void *)&stk[stksize]; stk++;
    *stk++   = (OS_STK_TYPE)(task);                       /* PC : task start address              */
    *stk++   = (OS_STK_TYPE)h;                            /* simulate function call PC (22..16)   */
    *stk++   = (OS_STK_TYPE)(task);                       /* PC : task start address INT          */
    *stk++   = (OS_STK_TYPE)(x | h);                      /* SR Low, CORCON IPL3 and PC (22..16)  */
    *stk++   = (OS_STK_TYPE)Tdata;                        /* W0 : argument                        */
    *stk++   = 0x1111;                                    /* W1                                   */
    *stk++   = 0x2222;                                    /* W2                                   */
    *stk++   = 0x3333;                                    /* W3                                   */
    *stk++   = 0x4444;                                    /* W4                                   */
    *stk++   = 0x5555;                                    /* W5                                   */
    *stk++   = 0x6666;                                    /* W6                                   */
    *stk++   = 0x7777;                                    /* W7                                   */
    *stk++   = 0x8888;                                    /* W8                                   */
    *stk++   = 0x9999;                                    /* W9                                   */
    *stk++   = 0xAAAA;                                    /* W10                                  */
    *stk++   = 0xBBBB;                                    /* W11                                  */
    *stk++   = 0xCCCC;                                    /* W12                                  */
    *stk++   = 0xDDDD;                                    /* W13                                  */
    *stk++   = 0xEEEE;                                    /* W14                                  */

    *stk++   = TBLPAG;                                    /* Data Table Page Address Register     */
#ifdef __HAS_EDS__
    *stk++   = DSRPAG;
    *stk++   = DSWPAG;
#else
    *stk++   = PSVPAG;                                    /* Program Space Visability Register    */
#endif
    *stk++   = RCOUNT;                                    /* Repeat Loop Counter Register         */
    *stk++   = 0;                                         /* SR enable all interrupt, clear flags */
    *stk++   = CORCON;                                    /* Core Control Register                */ 

#if OS_STK_CHECK_EN
    *stk++   = (OS_STK_TYPE)*pstk;                        /* Stack Pointer Limit Register         */
#else
    *stk++   = (OS_STK_TYPE)-1;                           /* unused                               */
#endif
    // ---------------------
    // pstk was updated and is now the ascending end-of-stack
    return stk;
}
