/*************************************************************
*
* Description:
*	PIC32MM - Example program using MPLAB-X
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#define FOSC   24000000U    // behind the PLL

// config of the project - add following include-pathes to your project
// "<your install directory> \Microchip\xc32\v2.50\pic32-libs" 

#include <sys/attribs.h>

#include "../../inc/pC_OS.h"

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define CORE_TICK_DIV         (FOSC / OS_TIMER_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void SysTick_init(void)
{
    __builtin_mtc0(11,0,(__builtin_mfc0(9,0) + CORE_TICK_DIV));
    __builtin_mtc0(9,0,0);
    IPC0bits.CTIP   = 2;  // Core Timer Priority 2
    IPC0bits.CTIS   = 0;  // Core Timer Sub Priority 0
    IEC0bits.CTIE   = 1;  // Core Timer Interrupt Enable
    INTCONbits.MVEC = 1;  // enable device multi-vector interrupts
}

void SysTick_update(void)
{
    IFS0bits.CTIF = 0;
    __builtin_mtc0(11,0,(__builtin_mfc0(9,0) + CORE_TICK_DIV));
}

void LED_init(void)
{
    TRISBbits.TRISB7 = 0; // RB7 is not analog default
    LATBCLR = 0x80;       // RB7 LED on
}

void LED_toggle(void)
{
    LATBINV = 0x80;       // RB7 LED toggle
}

void SystemInit(void)
{
    SYSKEY  = 0x0;
    SYSKEY  = 0xAA996655;
    SYSKEY  = 0x556699AA;
    OSCCONbits.FRCDIV = 0;
    OSCCONbits.NOSC   = 0;
    OSCCONbits.COSC   = 0;
    OSCCONbits.OSWEN  = 0;

    SPLLCON = 0x10080;
    RSWRST  = 0x0;
    RNMICON = 0x0;

    SYSKEY  = 0x0;
    while (OSCCONbits.OSWEN == 1);
    
    IFS0CLR = _IFS0_CS0IF_MASK;
    IPC0CLR = _IPC0_CS0IP_MASK;
    IPC0SET = ( 1 << _IPC0_CS0IP_POSITION );  // prio
    IEC0CLR = _IEC0_CS0IE_MASK;
    IEC0SET = 1 << _IEC0_CS0IE_POSITION;
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // PIC32 pC/OS ISR-code on top of the spezific vector-entry code
{
    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;

    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1,  500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer,      30, LedTimerCB,    NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif
    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
