;***************************************************************
;*                       pC/OS 1.87b                           *
;*                      a FREE - RTOS                          *
;*                                                             *
;*                 based on the C/OS 1.00                     *
;*                   by  Jean J. Labrosse                      *
;*                                                             *
;*    changed by Frank Goetze  -  www.embedded-os.de           *
;***************************************************************
;*                      OS_RL78_a.asm                          *
;*                Renesas RL78 Specific Code                   *
;*                        using IAR                            *
;***************************************************************


;***************************************************************
;                     MACRO DEFINITIONS
;***************************************************************
PUSHALL MACRO
        PUSH   RP0
        PUSH   RP1
        PUSH   RP2
        PUSH   RP3
        DW     0xFCAE
        DB     0xC1
        ENDM
        
POPALL  MACRO
        DB     0xC0
        DW     0xFCBE
        POP    RP3
        POP    RP2
        POP    RP1
        POP    RP0
        ENDM 
         
         
    EXTERN  _OSTCBCur
    EXTERN  _OSTCBHighRdy
    EXTERN  _OSIntNesting
    EXTERN  _OSTickISR
    EXTERN  _OSIntExit

    RSEG `.text`:CODE:ROOT
    
;***************************************************************
;                     START MULTITASKING
;                  void OSStartHighRdy(void)
;***************************************************************
    PUBLIC _OSStartHighRdy
_OSStartHighRdy:
        MOVW    RP1, _OSTCBHighRdy          ; address of OSTCBHighRdyPtr in RP1
        MOVW    RP0, 0x0000[RP1]            ; RP0 = OSTCBHighRdyPtr->OSTCBStkPtr
        MOVW    SP, RP0                     ; stack pointer = RP0
        POPALL                              ; restore all processor registers from new task's stack
        RETI                                ; return from interrupt 

;***************************************************************
;           PERFORM A CONTEXT SWITCH (From task level)
;                      void OSCtxSw(void)
;***************************************************************
;                       not needed here
;***************************************************************



;***************************************************************
;           PERFORM A CONTEXT SWITCH (From an ISR)
;                    void OSIntCtxSw(void)
;***************************************************************
    PUBLIC _OSIntCtxSw
_OSIntCtxSw:
        MOVW    RP0, _OSTCBHighRdy          ; get address of OSTCBHighRdy
        MOVW    _OSTCBCur, RP0              ; OSTCBCur = OSTCBHighRdy
        MOVW    RP1, RP0
        MOVW    RP0, 0x0000[RP1]            ; RP0 = OSTCBHighRdy
        MOVW    SP, RP0                     ; stack pointer = RP0
        POPALL                              ; restore all processor registers from new task's stack
        RETI                                ; return from interrupt 

;***************************************************************
;                    HANDLE CONTEXT-SWITCH
;                     void OSCtxSw_ISR(void)
;***************************************************************
;   ==> this must be hanged into INT-table 
;       offset-0x007E 
;***************************************************************
    PUBLIC _OSCtxSw_ISR
_OSCtxSw_ISR:
 #if (__VER__ < 300)
        PUSHALL                             ; Save processor registers on the stack
 #endif
                                            ; save current task's stack pointer into current task's OS_TCB
        MOVW    RP2, _OSTCBCur              ; OSTCBCur in RP2
        MOVW    RP0, SP
 #if (__VER__ > 300)
        ADDW    RP0, #4                     ; sub call to this code from stack
 #endif
        MOVW    [RP2], RP0                  ; OSTCBCur = SP
        MOVW    RP0, _OSTCBHighRdy          ; get address of OSTCBHighRdy
        MOVW    _OSTCBCur, RP0              ; OSTCBCur = OSTCBHighRdy
        MOVW    RP1, RP0
        MOVW    RP0, 0x0000[RP1]            ; RP0 = OSTCBHighRdy
        MOVW    SP, RP0                     ; stack pointer = RP0
        POPALL                              ; restore all processor registers from new task's stack
        RETI                                ; return from interrupt

;***************************************************************
;                       HANDLE TICK ISR
;                     void OSTickISR(void)
;***************************************************************
;   ==> this must be hanged into INT-table 
;       offset-0x0038 
;***************************************************************
    PUBLIC _OSTick_ISR
_OSTick_ISR: 
 #if (__VER__ < 300)
        PUSHALL                             ; Save processor registers on the stack
 #endif
        INC     _OSIntNesting               ; increment OSIntNesting
        CMP     _OSIntNesting, #1           ; if OSIntNesting != 1
        BNZ     _OSTick_ISR1                ; jump to OSTick_ISR1
                                            ; else
        MOVW    RP2, _OSTCBCur              ; OSTCBCurin RP2
        MOVW    RP0, SP
 #if (__VER__ > 300)
        ADDW    RP0, #4                     ; sub call to this code from stack
 #endif
        MOVW    [RP2], RP0                  ; OSTCBCur = SP
_OSTick_ISR1:
        CALL    _OSTickISR                  ; call OSTickISR()
        CALL    _OSIntExit                  ; call OSIntExit()
        POPALL                              ; restore all processor registers from stack
        RETI                                ; return from interrupt 

;***************************************************************
;                    SPECIAL IRQ-ISR (ALL)
;                     void OSirqISR(void)
;***************************************************************
;                       not needed here
;***************************************************************


;***************************************************************
;                    SPECIAL INT-SAVE CALLS
;                  OS_STK_TYPE OSEnterCrit(void)
;                  void OSExitCrit(OS_STK_TYPE)
;***************************************************************
;                       not needed here
;***************************************************************



#if (__VER__ > 300)

 #warning "assembler vectors are not working!"
 ; - on assembler versions 1.xx the "COMMON + ORG" version sets the vectors
 ; - on assembler versions 2.xx (2.21 tested) the "ASEGN" version sets the vectors
 ; - on assembler version  3.1x vectors can't be set in assembler 
 ;   the C-feature " #pragma vector = 0x7E
 ;                   __interrupt void OSshedvect(void) " 
 ;   forwarded to "_OSCtxSw_ISR" requires changes in the assembler code

#else
 #if (__VER__ > 200)
 
    ASEGN  `.intvec`:CODE:ROOT,038H
    DATA16
    DC16   _OSTick_ISR    
    ASEGN  `.intvec`:CODE:ROOT,07EH
    DATA16
    DC16   _OSCtxSw_ISR
    
 #else
 
    RSEG    INTVEC:CODE
    COMMON  INTVEC:CODE:ROOT(1)
    ORG     0x0038
    DW      _OSTick_ISR
    ORG     0x007E
    DW      _OSCtxSw_ISR

 #endif
#endif
;***************************************************************
    END


