/*************************************************************
*
* Description:
*	Rabbit 3400 - Example program using Softools WinIDE
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   512

// #define PDIUSBD12_TEST

#ifdef PDIUSBD12_TEST
 #define USB_CMD_ADDR     0xA001U
 #define USB_DATA_ADDR    0xA000U
 #define USB_D12_ID       0xFD
 #define USB_WAIT_READY   { U16 delay; delay = 0; while(delay < 10) delay++; }
#endif

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];

U08         pool[2048];

OS_MUX Mutex1;
OS_EVG Events1;

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void _fastint TimerISR(void);
#ifdef PDIUSBD12_TEST
 void ExtIO_Init(void);
#endif

void main()
{
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), 2048);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);

    OS_Start();
}

#ifdef PDIUSBD12_TEST
 U08 volatile ine_ex( U16 addr )
 {
    _HL = addr;
 #asm
    push    ip
    ipset   3
    ioe     ld a,(hl)                      ; read from 0xA000 (data-port of PDIUSBD12)
    pop     ip                             ; this opcode-fetch must be located on an odd address
 #endasm                                   ; for additional address hold-time on 0xA000 (data-port of PDIUSBD12)
    return (_A);
 }

 void ExtIO_Init(void)
 {
    // Initialize Port E-5 as CS for ext. IO
    enableIObus();
    ioSetBitI(PEDDR, 5);                    // set PE5 as Output
    ioSrOutI(IB5CR, 0xC8);                  // set to CS with 1WS low-active (permit write)
    ioSrSetBitI(PEFR, 5);                   // set to IO-strobe controlled by IB5CR
    ioSrOutI(I1CR, 0x07);                   // enable external INT1 on PE1, falling edge, priority 3
 }
#endif

void LED_PortDInit(void)
{
    // Initialize Port D
    ioSrOutI( PDCR, 0 );                    // 0 for pclk/2
    ioSrOutI( PDFR, 0 );                    // 0 for normal function
    ioSrOutI( PDDCR, 0xC0 );                // 6&7 open drain
    ioSrOutI( PDDR, 0xC0 );                 // 6&7 output high
    ioSrOutI( PDDDR, 0xC0 );                // 7&6&2 output, rest input
}

void AppTickInit(void)
{
    WDT_DISABLE();
    stopTimer();
    startTimer(10, TimerISR, 2);
    ipset1();
}

void Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    Tdata = Tdata;
    i=0;
#ifdef PDIUSBD12_TEST
    ExtIO_Init();
#endif
    LED_PortDInit();
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        while(i<2500) i++;                              // no Task-switch to MUTEX
        i=0;
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(1);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(1);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}


void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // suspending
        if(!returnOk) {
            while(i<2500) i++;                          // no Taskswitch to Task1 or Task3
            i=0;
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(1);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

void Task3(void *Tdata)
{
#ifdef PDIUSBD12_TEST
    U08  ID[2];
#endif
    U16  i;
    U08  LEDs = ~0x80;                     // ~1 for 7th LED

    Tdata = Tdata;
    i=0;
    while(1) {
#ifdef PDIUSBD12_TEST
        ID[0] = 0; ID[1] = 0;
        oute(USB_CMD_ADDR, USB_D12_ID); USB_WAIT_READY;     // PDIUSBD12 Chip-ID
        ID[0]=ine_ex(USB_DATA_ADDR); USB_WAIT_READY;
        ID[1]=ine_ex(USB_DATA_ADDR); USB_WAIT_READY;
//        if(ID[0] != 0x12 || ID[1] != 0x10)
//            while(1);
#endif
        outi(PDDR, LEDs);                   // Lite one LED
        OS_TimeDly(1);                      // test timer-wakeup
        LEDs ^= 0xC0;                       // Toggle them
        i++;
    }
}

void TimerISR(void)
{
    OSTickISR();
}
