/*************************************************************
*
* Description:
*   SAMD51 - Example program using Keil / IAR
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define SAMD51_TACHYON        0
#define SAMD51_ITSYBITSY_M4   1

#define SAMD51_TYPE           SAMD51_ITSYBITSY_M4

#define PORTA         0                       // user-LED definition
#define LED_GROUP     PORTA
#if (SAMD51_TYPE == SAMD51_TACHYON)           // -- TACHYON board
 #define LED_PORT     PORT_PA27
 #define LED_PIN      PIN_PA27
#endif
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)      // -- Adafruit ItsyBitsy M4 Express
 #define LED_PORT     PORT_PA22
 #define LED_PIN      PIN_PA22
#endif

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

/***************************************************************/

void SysTick_init(void)
{
    NVMCTRL->CTRLA.reg |= NVMCTRL_CTRLA_RWS(5);
    GCLK->CTRLA.bit.SWRST = 1;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_SWRST) _NOP();
    GCLK->GENCTRL[3].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_OSCULP32K) | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL3) _NOP();
    GCLK->GENCTRL[0].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_OSCULP32K) | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL0) _NOP();
    OSCCTRL->DFLLCTRLA.reg = 0;
    //GCLK->PCHCTRL[OSCCTRL_GCLK_ID_DFLL48].reg = (1 << GCLK_PCHCTRL_CHEN_Pos) | GCLK_PCHCTRL_GEN(GCLK_PCHCTRL_GEN_GCLK3_Val);
    OSCCTRL->DFLLMUL.reg = OSCCTRL_DFLLMUL_CSTEP(1) | OSCCTRL_DFLLMUL_FSTEP(1) | OSCCTRL_DFLLMUL_MUL(0);
    while (OSCCTRL->DFLLSYNC.reg & OSCCTRL_DFLLSYNC_DFLLMUL) _NOP();
    OSCCTRL->DFLLCTRLB.reg = 0;
    while (OSCCTRL->DFLLSYNC.reg & OSCCTRL_DFLLSYNC_DFLLCTRLB) _NOP();
    OSCCTRL->DFLLCTRLA.reg |= OSCCTRL_DFLLCTRLA_ENABLE;
    while (OSCCTRL->DFLLSYNC.reg & OSCCTRL_DFLLSYNC_ENABLE) _NOP();
    OSCCTRL->DFLLVAL.reg = OSCCTRL->DFLLVAL.reg;
    while (OSCCTRL->DFLLSYNC.bit.DFLLVAL) _NOP();
    OSCCTRL->DFLLCTRLB.reg = OSCCTRL_DFLLCTRLB_WAITLOCK | OSCCTRL_DFLLCTRLB_CCDIS | OSCCTRL_DFLLCTRLB_USBCRM ;
    while (!OSCCTRL->STATUS.bit.DFLLRDY) _NOP();
    GCLK->GENCTRL[5].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DFLL_Val) | GCLK_GENCTRL_GENEN | GCLK_GENCTRL_DIV(24u);
    while (GCLK->SYNCBUSY.bit.GENCTRL5) _NOP();
    GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL0].reg = (1 << GCLK_PCHCTRL_CHEN_Pos) | GCLK_PCHCTRL_GEN(GCLK_PCHCTRL_GEN_GCLK5_Val);
    OSCCTRL->Dpll[0].DPLLRATIO.reg = OSCCTRL_DPLLRATIO_LDRFRAC(0x00) | OSCCTRL_DPLLRATIO_LDR(59);
    while (OSCCTRL->Dpll[0].DPLLSYNCBUSY.bit.DPLLRATIO) _NOP();
    OSCCTRL->Dpll[0].DPLLCTRLB.reg = OSCCTRL_DPLLCTRLB_REFCLK_GCLK | OSCCTRL_DPLLCTRLB_LBYPASS;
    OSCCTRL->Dpll[0].DPLLCTRLA.reg = OSCCTRL_DPLLCTRLA_ENABLE;
    while ((OSCCTRL->Dpll[0].DPLLSTATUS.bit.CLKRDY == 0) || (OSCCTRL->Dpll[0].DPLLSTATUS.bit.LOCK == 0)) _NOP();
    GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL1].reg = (1 << GCLK_PCHCTRL_CHEN_Pos) | GCLK_PCHCTRL_GEN(GCLK_PCHCTRL_GEN_GCLK5_Val);
    OSCCTRL->Dpll[1].DPLLRATIO.reg = OSCCTRL_DPLLRATIO_LDRFRAC(0x00) | OSCCTRL_DPLLRATIO_LDR(49);
    while (OSCCTRL->Dpll[1].DPLLSYNCBUSY.bit.DPLLRATIO) _NOP();
    OSCCTRL->Dpll[1].DPLLCTRLB.reg = OSCCTRL_DPLLCTRLB_REFCLK_GCLK | OSCCTRL_DPLLCTRLB_LBYPASS;
    OSCCTRL->Dpll[1].DPLLCTRLA.reg = OSCCTRL_DPLLCTRLA_ENABLE;
    while ((OSCCTRL->Dpll[1].DPLLSTATUS.bit.CLKRDY == 0) || (OSCCTRL->Dpll[1].DPLLSTATUS.bit.LOCK == 0)) _NOP();
    GCLK->GENCTRL[1].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DFLL_Val) | GCLK_GENCTRL_IDC | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL1) _NOP();
    GCLK->GENCTRL[2].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DPLL1_Val) | GCLK_GENCTRL_IDC | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL2) _NOP();
    GCLK->GENCTRL[0].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DPLL0) | GCLK_GENCTRL_IDC | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL0) _NOP();
    MCLK->CPUDIV.reg = MCLK_CPUDIV_DIV_DIV1;
    SUPC->VREG.bit.SEL = 0; 
    // ---------------------
    SystemCoreClockUpdate();
    if (SysTick_Config(120000000 / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
}

void SysTick_update(void)
{
}

void LED_init(void)
{
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
}

void LED_toggle(void)
{
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical Cortex-M4 pC/OS ISR
{
    OS_IntEnter();

    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
    OS_IntExit();
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;
 
    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
