;/***************************************************************
;/*                       pC/OS 1.87b                           *
;/*                      a FREE - RTOS                          *
;/*                                                             *
;/*                 based on the C/OS 1.00                     *
;/*                   by  Jean J. Labrosse                      *
;/*                                                             *
;/*    changed by Frank Goetze  -  www.embedded-os.de           *
;/***************************************************************
;/*                       OS_SAML11_a.s                         *
;/*            ARMv8 Cortex-M23 Specific Code (with TZ)         *
;/*                       using Keil                            *
;/***************************************************************

OS_NVIC_SHPR3             EQU     0xE000ED20
OS_NVIC_SHPR3_PENDSV      EQU     0x00C00000
OS_NVIC_ICSR              EQU     0xE000ED04
OS_NVIC_ICSR_PENDSV       EQU     0x10000000

        IMPORT  OSTCBCur
        IMPORT  OSTCBHighRdy

    PRESERVE8
    THUMB
    AREA    |.text|, CODE, READONLY

;/***************************************************************
;/                     START MULTITASKING
;/                  void OSStartHighRdy(void)
;/***************************************************************
    EXPORT OSStartHighRdy
    THUMB
OSStartHighRdy
        LDR     R0, =OS_NVIC_SHPR3         ; set PendSV-exception priority lowest
        LDR     R1, [R0]
        LDR     R2, =OS_NVIC_SHPR3_PENDSV
        ORRS    R1, R1, R2
        STR     R1, [R0]
        MOV     R0, #0                     ; set PSP for initial PendSV-exception
        MSR     PSP, R0
        LDR     R0, =OS_NVIC_ICSR          ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_ICSR_PENDSV
        STR     R1, [R0]
        CPSIE   I                          ; enable interrupts
        BX      LR                         ; (for debugging)

;/***************************************************************
;/           PERFORM A CONTEXT SWITCH (From task level)
;/                      void OSCtxSw(void)
;/***************************************************************
    EXPORT OSCtxSw
    THUMB
OSCtxSw
        LDR     R0, =OS_NVIC_ICSR          ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_ICSR_PENDSV
        STR     R1, [R0]
        BX      LR

;/***************************************************************
;/           PERFORM A CONTEXT SWITCH (From an ISR)
;/                    void OSIntCtxSw(void)
;/***************************************************************
    EXPORT OSIntCtxSw
    THUMB
OSIntCtxSw
        LDR     R0, =OS_NVIC_ICSR          ; trigger PendSV-exception (executed after leaving interrupt) 
        LDR     R1, =OS_NVIC_ICSR_PENDSV
        STR     R1, [R0]
        BX      LR

;/***************************************************************
;/                    HANDLE CONTEXT-SWITCH
;/                   void OSPendSV_ISR(void)
;/***************************************************************
;/   ==> this must be hanged into INT-table 
;/       offset-14 (PendSV handler)
;/***************************************************************
    EXPORT OSPendSV_ISR
    THUMB
OSPendSV_ISR
        MRS     R0, PSP                    ; check initial PendSV-exception
        CBZ     R0, OSPendISR_nosave       ; skip register-save the first time
        SUBS    R0, R0, #0x10
        STMIA   R0!, {R4-R7}               ; save r4-r7
        MOV     R4, R8
        MOV     R5, R9
        MOV     R6, R10
        MOV     R7, R11
        SUBS    R0, R0, #0x20
        STMIA   R0!, {R4-R7}               ; save r8-r11
        SUBS    R0, R0, #0x10
        
        SUBS    R0, R0, #0x04
        MOV     R1, LR
        STR     R1, [R0]                   ; save LR
        
        SUBS    R0, R0, #0x04
        MRS     R1, PSPLIM
        STR     R1, [R0]                   ; save PSPLIM

        LDR     R1, =OSTCBCur              ; save stack ptr in TCB
        LDR     R1, [R1]
        STR     R0, [R1]
OSPendISR_nosave
        LDR     R2, =OSTCBHighRdy          ; get TCB of highest priority task
        LDR     R1, =OSTCBCur              ; save as current 
        LDR     R2, [R2]
        STR     R2, [R1]
        LDR     R0, [R2]                   ; get new stack-ptr

        LDMIA   R0!, {R1}                  ; restore PSPLIM
        MSR     PSPLIM, R1                 ; load PSPLIM with new ptr

        LDMIA   R0!, {R1}                  ; restore LR
        
        LDMIA   R0!, {R4-R7}               ; restore r8-r11
        MOV     R8, R4
        MOV     R9, R5
        MOV     R10, R6
        MOV     R11, R7
        LDMIA   R0!, {R4-R7}               ; restore r4-r7
        MSR     PSP, R0                    ; load PSP with new stack-ptr
        BX      R1

;/***************************************************************
;/                       HANDLE TICK ISR
;/                     void OSTickISR(void)
;/***************************************************************
;/                       not needed here
;/***************************************************************

;/***************************************************************
;/                    SPECIAL IRQ-ISR (ALL)
;/                     void OSirqISR(void)
;/***************************************************************
;/                       not needed here
;/***************************************************************


;/***************************************************************
;/                    SPECIAL INT-SAVE CALLS
;/                  OS_STK_TYPE OSEnterCrit(void)
;/                  void OSExitCrit(OS_STK_TYPE)
;/***************************************************************
    EXPORT OSEnterCrit
    THUMB
OSEnterCrit
        MRS     R0, PRIMASK
        CPSID   I
        BX      LR                          ; return R0

    EXPORT OSExitCrit
    THUMB
OSExitCrit
        MSR     PRIMASK, R0
        BX      LR                          ; return

;/***************************************************************
    ALIGN 4
    END


