/*
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ----------------------------------------------------------------------
 *
 * main_s.c      Code template for secure main function 
 *
 * Version 1.0
 *    Initial Release
 *---------------------------------------------------------------------------*/
 
/* Use CMSE intrinsics */
#include <arm_cmse.h>
 
#include "RTE_Components.h"
#include CMSIS_device_header

#include "stm32l5xx_hal_rcc.h"
#include "stm32l5xx_hal_gpio.h"

/* TZ_START_NS: Start address of non-secure application */
#ifndef TZ_START_NS
#define TZ_START_NS (0x08040000U)
#endif

/* Default process stack size */
#ifndef PROCESS_STACK_SIZE
#define PROCESS_STACK_SIZE 512U
#endif

/* Default process stack */
static uint64_t ProcessStack[PROCESS_STACK_SIZE/8U];

/* typedef for non-secure callback functions */
typedef void (*funcptr_void) (void) __attribute__((cmse_nonsecure_call));

/* Secure main() */
extern void BOARD_InitTrustZone(void);
int main(void) {
    funcptr_void NonSecure_ResetHandler;

    /* Add user setup code for secure part here*/
    BOARD_InitTrustZone();

    /* Set non-secure vector table location */
    SCB_NS->VTOR = (uint32_t)(TZ_START_NS);

    /* Set non-secure main stack (MSP_NS) */
    __TZ_set_MSP_NS(*((uint32_t *)(TZ_START_NS)));

    /* Set default PSP, PSPLIM and privileged Thread Mode using PSP */
    __set_PSPLIM((uint32_t)ProcessStack);
    __set_PSP   ((uint32_t)ProcessStack + PROCESS_STACK_SIZE);
    __set_CONTROL(0x02U);

    /* allow NS use of GPIO-A - LED PA9 */
    __HAL_RCC_GPIOA_CLK_ENABLE();
    CLEAR_BIT(GPIOA_S->SECCFGR, GPIO_PIN_9);

    /* Get non-secure reset handler */
    NonSecure_ResetHandler = (funcptr_void)(*((uint32_t *)((TZ_START_NS) + 4U)));
 
    /* Start non-secure state software application */
    NonSecure_ResetHandler();

    /* Non-secure software does not return, this code is not executed */
    while (1) {
        __NOP();
    }
}
