/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_his.c                         *
*                    HISTORY SERVICE                       *
***********************************************************/

#if  OS_HIS_EN
/*
************************************************************
*                GLOBAL AND LOCAL VARIABLES                *
************************************************************
*/
    OS_H            OSHTable[OS_MAX_HISTORY];
    U08             OSHWriteIndex;
    U08             OSHEntryCount;
    U08             OSHReadIndex;
/*
************************************************************
*                    POST TO HISTORY-TABLE                 *
************************************************************
*/
U08 OS_HistoryPost(U32 param1, U32 param2) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_H        *pointer;

    OS_ENTER_CRITICAL();
    pointer = &OSHTable[OSHWriteIndex];
    pointer->OSHPara1 =  param1;
    pointer->OSHPara2 =  param2;
    pointer->OSHPrio  =  OSTCBCur->OSTCBPrio;
    pointer->OSHTime  =  OSSystemCnt;
    OSHWriteIndex++;
    if (OSHWriteIndex >= OS_MAX_HISTORY)
        OSHWriteIndex =  0;
    if (OSHEntryCount < OS_MAX_HISTORY)
        OSHEntryCount++;
    else
        OSHReadIndex = OSHWriteIndex;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                    READ A HISTORY-ENTRY                  *
************************************************************
*/
U08 OS_HistoryRead(U32 *param1, U32 *param2, U08 *prio, U32 *time) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_H        *pointer;

    OS_ENTER_CRITICAL();
    if (OSHEntryCount) {
        pointer = &OSHTable[OSHReadIndex];
        *param1 = pointer->OSHPara1;
        *param2 = pointer->OSHPara2;
        *prio   = pointer->OSHPrio;
        *time   = pointer->OSHTime;
        OSHReadIndex++;
        if (OSHReadIndex >= OS_MAX_HISTORY)
            OSHReadIndex =  0;
        OSHEntryCount--;
    } else {
        OS_EXIT_CRITICAL();
        return (OS_HIS_END);
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
#endif /* OS_HIS_EN */

/************************* END ****************************/


