/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the µC/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_icc.c                         *
*                  ICC AMP-PIPE SERVICE                    *
***********************************************************/

#if OS_ICC_EN

void OS_IccEventHandler(void);

#ifdef _OS_STM32H745_
 #include "../STM32H745/OS_ICC_STM32H745.c"    // specials for STM32H745
#endif
#ifdef _OS_LPC43xx_
 #include "../LPC4337/OS_ICC_LPC4337.c"        // specials for LPC4337
#endif

OS_ICC_TX Icc_Tx;
OS_ICC_RX Icc_Rx;

/*
************************************************************
*               EVENTHANDLER ICC AMP-PIPE                  *
************************************************************
*/
void OS_IccEventHandler(void)
{
    U08            x, y, bitx, bity, p;

    if ((Icc_Tx.OSIccShared != NULL) && (Icc_Tx.OSIccShared->OSIccEntries < OS_ICC_DEEP)) {
        if (Icc_Tx.OSPBGrp != 0) {
            y                = OSUnMapTbl[Icc_Tx.OSPBGrp];
            x                = OSUnMapTbl[Icc_Tx.OSPBTbl[y]];
            bity             = OSMapTbl[y];
            bitx             = OSMapTbl[x];
            if ((U08)(Icc_Tx.OSPBTbl[y] &= ~(bitx)) == 0)
                Icc_Tx.OSPBGrp &= ~(bity);
            p                           = (y << 3) + x;
            OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_ICC);
            OSTCBPrioTbl[p]->OSTCBDly   = 0;
            OSRdyGrp                   |= bity;
            OSRdyTbl[y]                |= bitx;
        }
    }
    if ((Icc_Rx.OSIccShared != NULL) && (Icc_Rx.OSIccShared->OSIccEntries > 0)) {
        if (Icc_Rx.OSPFGrp != 0) {
            y                = OSUnMapTbl[Icc_Rx.OSPFGrp];
            x                = OSUnMapTbl[Icc_Rx.OSPFTbl[y]];
            bity             = OSMapTbl[y];
            bitx             = OSMapTbl[x];
            if ((U08)(Icc_Rx.OSPFTbl[y] &= ~(bitx)) == 0)
                Icc_Rx.OSPFGrp &= ~(bity);
            p                           = (y << 3) + x;
            OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_ICC);
            OSTCBPrioTbl[p]->OSTCBDly   = 0;
            OSRdyGrp                   |= bity;
            OSRdyTbl[y]                |= bitx;
        }
    }        
}
/*
************************************************************
*                INITIALIZE ICC AMP-PIPE                   *
************************************************************
*/
U08 OS_IccInit(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          i;

    OS_ENTER_CRITICAL();
    Icc_Tx.OSIccShared               = IccTxS;
    Icc_Tx.OSIccShared->OSIccData    = IccTxDataS;
    Icc_Tx.OSIccShared->OSIccIn      = 0x00;
    Icc_Tx.OSIccShared->OSIccOut     = 0x00;
    Icc_Tx.OSIccShared->OSIccEntries = 0;
    Icc_Tx.OSPBGrp                   = 0x00;
    for (i = 0; i < OS_PRIO_TBL; i++)
        Icc_Tx.OSPBTbl[i]  = 0x00;
    Icc_Rx.OSIccShared               = IccRxS;
    if (Icc_Rx.OSIccShared->OSIccData != IccRxDataS)
        Icc_Rx.OSIccShared->OSIccEntries = 0;
    Icc_Rx.OSPFGrp                   = 0x00;
    for (i = 0; i < OS_PRIO_TBL; i++)
        Icc_Rx.OSPFTbl[i]  = 0x00;
    OS_EXIT_CRITICAL();
    ICC_IRQEnable();
    return (OS_NO_ERR);
}
/*
************************************************************
*              GET INFO OF ICC AMP-PIPE(s)                 *
************************************************************
*/
U08 OS_IccInfo(U16 *size, U08 *deep, U08 *usedRx, U08 *usedTx) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    *size   = OS_ICC_SIZE;
    *deep   = OS_ICC_DEEP;
    *usedRx = (Icc_Rx.OSIccShared != NULL)? Icc_Rx.OSIccShared->OSIccEntries : 0;
    *usedTx = Icc_Tx.OSIccShared->OSIccEntries;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*             CLEAR ALL DATA OF ICC AMP-PIPE               *
************************************************************
*/
U08 OS_IccClear(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE    OS_sr;
#endif
    U08            x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    Icc_Tx.OSIccShared->OSIccIn      = 0x00;
    Icc_Tx.OSIccShared->OSIccOut     = 0x00;
    Icc_Tx.OSIccShared->OSIccEntries = 0;
    if (Icc_Tx.OSPBGrp != 0) {
        y                = OSUnMapTbl[Icc_Tx.OSPBGrp];
        x                = OSUnMapTbl[Icc_Tx.OSPBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(Icc_Tx.OSPBTbl[y] &= ~(bitx)) == 0)
            Icc_Tx.OSPBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_ICC);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 PEND ON A ICC AMP-PIPE                   *
************************************************************
*/
U08 OS_IccPend(U08 OS_HUGE *msg, U16 *lng, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U16            i;
    U08            x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if ((Icc_Rx.OSIccShared == NULL) || (Icc_Rx.OSIccShared->OSIccEntries == 0)) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_ICC_NODATA);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = (void *)&Icc_Rx;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_ICC;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        Icc_Rx.OSPFTbl[y] |= bitx;
        Icc_Rx.OSPFGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if (((U08)(OSTCBCur->OSTCBStat & OS_STAT_ICC)) && ((Icc_Rx.OSIccShared == NULL) || (Icc_Rx.OSIccShared->OSIccEntries == 0))){
            if ((U08)(Icc_Rx.OSPFTbl[y] &= ~(bitx)) == 0)
                Icc_Rx.OSPFGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if ((Icc_Rx.OSIccShared == NULL) || (Icc_Rx.OSIccShared->OSIccEntries == 0)) {
            OS_EXIT_CRITICAL();
            return (OS_P_NODATA);
        }
    }
    y = Icc_Rx.OSIccShared->OSIccOut;
    Icc_Rx.OSIccShared->OSIccOut++;
    if (Icc_Rx.OSIccShared->OSIccOut == OS_ICC_DEEP)
        Icc_Rx.OSIccShared->OSIccOut = 0;
    OSLockNesting++;
    OS_EXIT_CRITICAL();
    *lng = (U16)(Icc_Rx.OSIccShared->OSIccData[y * OS_ICC_SIZE]) +
           (U16)(Icc_Rx.OSIccShared->OSIccData[y * OS_ICC_SIZE + 1]) * 256;
    for (i = 0; i < (*lng); i++)
        msg[i] = Icc_Rx.OSIccShared->OSIccData[y * OS_ICC_SIZE + sizeof(U16) + i];
    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    if (Icc_Rx.OSIccShared->OSIccEntries > 0) {
        y = Icc_Rx.OSIccShared->OSIccEntries;
        Icc_Rx.OSIccShared->OSIccEntries--;
        if (y >= OS_ICC_DEEP)
            ICC_Event();
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}

/*
************************************************************
*                 POST TO A ICC AMP-PIPE                   *
************************************************************
*/
U08 OS_IccPost(U08 OS_HUGE *msg, U16 len, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U16            i;
    U08            x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (len > (OS_ICC_SIZE - sizeof(U16))) {
        OS_EXIT_CRITICAL();
        return (OS_ICC_LEN_ERR);
    }
    if (Icc_Tx.OSIccShared->OSIccEntries >= OS_ICC_DEEP) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_ICC_FULL);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = (void *)&Icc_Tx;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_ICC;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((U08)(OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        Icc_Tx.OSPBTbl[y] |= bitx;
        Icc_Tx.OSPBGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if (OSTCBCur->OSTCBStat & OS_STAT_ICC) {
            if ((U08)(Icc_Tx.OSPBTbl[y] &= ~(bitx)) == 0)
                Icc_Tx.OSPBGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (Icc_Tx.OSIccShared->OSIccEntries >= OS_ICC_DEEP) {
            OS_EXIT_CRITICAL();
            return (OS_ICC_FULL);
        }
    }
    y = Icc_Tx.OSIccShared->OSIccIn;
    Icc_Tx.OSIccShared->OSIccIn++;
    if (Icc_Tx.OSIccShared->OSIccIn == OS_ICC_DEEP)
        Icc_Tx.OSIccShared->OSIccIn = 0;
    OSLockNesting++;
    OS_EXIT_CRITICAL();
    Icc_Tx.OSIccShared->OSIccData[y * OS_ICC_SIZE]     = (U08)(len%256);
    Icc_Tx.OSIccShared->OSIccData[y * OS_ICC_SIZE + 1] = (U08)(len/256);
    for (i = 0; i < len; i++)
        Icc_Tx.OSIccShared->OSIccData[y * OS_ICC_SIZE + sizeof(U16) + i] = msg[i];
    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    y = Icc_Tx.OSIccShared->OSIccEntries;
    Icc_Tx.OSIccShared->OSIccEntries++;
    if (y == 0)
        ICC_Event();
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}

#endif /* OS_ICC_EN */

/************************* END ****************************/
