/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_mbox.c                        *
*                   MESSAGEBOX SERVICE                     *
***********************************************************/

#if OS_MBOX_EN
/*
************************************************************
*                   INITIALIZE MAILBOX                     *
************************************************************
*/
U08 OS_MboxInit(OS_MBOX *pmbox) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          i;

    OS_ENTER_CRITICAL();
    pmbox->OSMboxMsg     = 0x00;
    pmbox->OSMboxValid   = FALSE;
    pmbox->OSMboxFGrp    = 0x00;
    pmbox->OSMboxBGrp    = 0x00;
    for (i = 0; i < OS_PRIO_TBL; i++) {
        pmbox->OSMboxFTbl[i] = 0x00;
        pmbox->OSMboxBTbl[i] = 0x00;
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     PEND ON MAILBOX                      *
************************************************************
*/
U08 OS_MboxPend(OS_MBOX *pmbox, void OS_FAR **msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (!pmbox->OSMboxValid) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_MBOX_NODATA);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pmbox;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_MBOX;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pmbox->OSMboxFTbl[y] |= bitx;
        pmbox->OSMboxFGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if ((OSTCBCur->OSTCBStat & OS_STAT_MBOX) && (!(pmbox->OSMboxValid))) {
            if ((U08)(pmbox->OSMboxFTbl[y] &= ~(bitx)) == 0)
                pmbox->OSMboxFGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (!pmbox->OSMboxValid) {
            OS_EXIT_CRITICAL();
            return (OS_MBOX_NODATA);
        }
    }
    *msg = pmbox->OSMboxMsg;
    pmbox->OSMboxValid  = FALSE;
    if (pmbox->OSMboxBGrp != 0) {
        y                = OSUnMapTbl[pmbox->OSMboxBGrp];
        x                = OSUnMapTbl[pmbox->OSMboxBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pmbox->OSMboxBTbl[y] &= ~(bitx)) == 0)
            pmbox->OSMboxBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_MBOX);
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                    POST TO A MAILBOX                     *
************************************************************
*/
U08 OS_MboxPost(OS_MBOX *pmbox, void OS_FAR *msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pmbox->OSMboxValid) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_MBOX_FULL);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pmbox;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_MBOX;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pmbox->OSMboxBTbl[y] |= bitx;
        pmbox->OSMboxBGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if ((OSTCBCur->OSTCBStat & OS_STAT_MBOX) && (pmbox->OSMboxValid)) {
            if ((U08)(pmbox->OSMboxBTbl[y] &= ~(bitx)) == 0)
                pmbox->OSMboxBGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return(OS_TIMEOUT);
        }
        if (pmbox->OSMboxValid) {
            OS_EXIT_CRITICAL();
            return (OS_MBOX_FULL);
        }
    }
    pmbox->OSMboxMsg    = msg;
    pmbox->OSMboxValid  = TRUE;
    if (pmbox->OSMboxFGrp != 0) {
        y                = OSUnMapTbl[pmbox->OSMboxFGrp];
        x                = OSUnMapTbl[pmbox->OSMboxFTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pmbox->OSMboxFTbl[y] &= ~(bitx)) == 0)
            pmbox->OSMboxFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_MBOX);
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*            ABBORT HIGHEST PENDER ON A MAILBOX            *
************************************************************
*/
U08 OS_MboxPendAbbort(OS_MBOX *pmbox) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pmbox->OSMboxFGrp != 0) {
        y                = OSUnMapTbl[pmbox->OSMboxFGrp];
        x                = OSUnMapTbl[pmbox->OSMboxFTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pmbox->OSMboxFTbl[y] &= ~(bitx)) == 0)
            pmbox->OSMboxFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_MBOX);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}
/*
************************************************************
*            ABBORT HIGHEST POSTER ON A MAILBOX            *
************************************************************
*/
U08 OS_MboxPostAbbort(OS_MBOX *pmbox) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pmbox->OSMboxBGrp != 0) {
        y                = OSUnMapTbl[pmbox->OSMboxBGrp];
        x                = OSUnMapTbl[pmbox->OSMboxBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pmbox->OSMboxBTbl[y] &= ~(bitx)) == 0)
            pmbox->OSMboxBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_MBOX);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*                DELETE A TASK FROM A MAILBOX              *
************************************************************
*/
void OSMboxDestroyTask(OS_MBOX *pmbox, U08 prio)
{
    U08          x, y, bitx, bity;

    y      = prio >> 3;
    x      = prio & 0x07;
    bity   = OSMapTbl[y];
    bitx   = OSMapTbl[x];
    if ((U08)(pmbox->OSMboxFTbl[y] &= ~(bitx)) == 0)
        pmbox->OSMboxFGrp &= ~(bity);
    if ((U08)(pmbox->OSMboxBTbl[y] &= ~(bitx)) == 0)
        pmbox->OSMboxBGrp &= ~(bity);
    OSTCBPrioTbl[prio]->OSTCBDly   =  0;
    OSTCBPrioTbl[prio]->OSTCBStat &= ~(OS_STAT_MBOX);
    OSRdyGrp                      |=  bity;
    OSRdyTbl[y]                   |=  bitx;
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_MBOX_EN */

/************************* END ****************************/


