/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_pipe.c                        *
*                      PIPE SERVICE                        *
***********************************************************/

#if OS_P_EN
/*
************************************************************
*                    INITIALIZE PIPE                       *
************************************************************
*/
U08 OS_PipeInit(OS_P *pp, void OS_HUGE *buffer, U16 size, U08 deep) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          i;

    if ((buffer != NULL) && (size != 0) && (deep != 0)) {}
    else if ((buffer == NULL) && (size == 0) && (deep == 0)) {}
    else return (OS_PARAM_ERR);
    OS_ENTER_CRITICAL();
    pp->OSPData     = buffer;
    pp->OSPIn       = 0x00;
    pp->OSPOut      = 0x00;
    pp->OSPSize     = size;
    pp->OSPDeep     = deep;
    pp->OSPEntries  = 0;
    pp->OSPData1    = (U08 OS_HUGE *)NULL;
    pp->OSPLenght1  = 0;
    pp->OSPFGrp     = 0x00;
    pp->OSPBGrp     = 0x00;
    for (i = 0; i < OS_PRIO_TBL; i++) {
        pp->OSPFTbl[i]  = 0x00;
        pp->OSPBTbl[i]  = 0x00;
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                   GET INFO OF PIPE                       *
************************************************************
*/
U08 OS_PipeInfo(OS_P *pp, U16 *size, U08 *deep, U08 *used, U08 *prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y;

    OS_ENTER_CRITICAL();
    *size = pp->OSPSize;
    *deep = pp->OSPDeep;
    *used = pp->OSPEntries;
    if (pp->OSPFGrp != 0) {
        y    = OSUnMapTbl[pp->OSPFGrp];
        x    = OSUnMapTbl[pp->OSPFTbl[y]];
       *prio = (y << 3) + x;
    } else
       *prio = 0;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 CLEAR ALL DATA OF PIPE                   *
************************************************************
*/
U08 OS_PipeClear(OS_P *pp) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE    OS_sr;
#endif
    OS_DP OS_HUGE *dp, *t;
    U08            x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    pp->OSPIn      = 0x00;
    pp->OSPOut     = 0x00;
    pp->OSPEntries = 0;
    pp->OSPData1   = (U08 OS_HUGE *)NULL;
    pp->OSPLenght1 = 0;
    if ((pp->OSPSize | pp->OSPDeep) == 0) {
        dp = (OS_DP OS_HUGE *)pp->OSPData;
        pp->OSPData = NULL;
        OS_EXIT_CRITICAL();
        while (dp != NULL) {
            t  = dp;
            dp = dp->OSDPNext;
            (void)OSMemFree((void OS_HUGE *)t, 0);
        }
        OS_ENTER_CRITICAL();
    }
    if (pp->OSPBGrp != 0) {
        y                = OSUnMapTbl[pp->OSPBGrp];
        x                = OSUnMapTbl[pp->OSPBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pp->OSPBTbl[y] &= ~(bitx)) == 0)
            pp->OSPBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_P);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     PEND ON A PIPE                       *
************************************************************
*/
U08 OS_PipePend(OS_P *pp, U08 OS_HUGE *msg, U16 *lng, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_DP OS_HUGE *dp;
    U16            i;
    U08            x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pp->OSPEntries == 0) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_P_NODATA);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pp;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_P;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pp->OSPFTbl[y] |= bitx;
        pp->OSPFGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if ((U08)(OSTCBCur->OSTCBStat & OS_STAT_P) && (pp->OSPEntries == 0)){
            if ((U08)(pp->OSPFTbl[y] &= ~(bitx)) == 0)
                pp->OSPFGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (pp->OSPEntries == 0) {
            OS_EXIT_CRITICAL();
            return (OS_P_NODATA);
        }
    }
    if ((pp->OSPData1 != (U08 OS_HUGE *)NULL) && (pp->OSPLenght1 != 0)) {
        if (pp->OSPEntries > 0)
            pp->OSPEntries--;
        *lng = pp->OSPLenght1;
        pp->OSPLenght1 = 0;
        OSLockNesting++;
        OS_EXIT_CRITICAL();
        for (i = 0; i < *lng; i++)
            msg[i] = pp->OSPData1[i];
        OS_ENTER_CRITICAL();
        if (OSLockNesting != 0)
            OSLockNesting--;
        pp->OSPData1   = (U08 OS_HUGE *)NULL;
        OS_EXIT_CRITICAL();
    } else {
        if ((pp->OSPSize | pp->OSPDeep) == 0) {
            dp = (OS_DP OS_HUGE *)pp->OSPData;
            pp->OSPData = (U08 OS_HUGE *)dp->OSDPNext;
            OS_EXIT_CRITICAL();
            *lng = dp->OSDPLenght;
            for (i = 0; i < (*lng); i++)
                msg[i] = dp->OSDPData[i];
            x = OSMemFree((void OS_HUGE *)dp, 0);
            if (x != OS_NO_ERR) return (x);
            OS_ENTER_CRITICAL();
        } else {
            y = pp->OSPOut;
            pp->OSPOut++;
            if (pp->OSPOut == pp->OSPDeep)
                pp->OSPOut = 0;
            OSLockNesting++;
            OS_EXIT_CRITICAL();
            *lng = (U16)(pp->OSPData[y * pp->OSPSize]) +
                   (U16)(pp->OSPData[y * pp->OSPSize + 1]) * 256;
            for (i = 0; i < (*lng); i++)
                msg[i] = pp->OSPData[y * pp->OSPSize + sizeof(U16) + i];
            OS_ENTER_CRITICAL();
            if (OSLockNesting != 0)
                OSLockNesting--;
        }
        if (pp->OSPEntries > 0)
            pp->OSPEntries--;
        if (pp->OSPBGrp != 0) {
            y                = OSUnMapTbl[pp->OSPBGrp];
            x                = OSUnMapTbl[pp->OSPBTbl[y]];
            bity             = OSMapTbl[y];
            bitx             = OSMapTbl[x];
            if ((U08)(pp->OSPBTbl[y] &= ~(bitx)) == 0)
                pp->OSPBGrp &= ~(bity);
            p                           = (y << 3) + x;
            OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_P);
            OSTCBPrioTbl[p]->OSTCBDly   = 0;
            OSRdyGrp                   |= bity;
            OSRdyTbl[y]                |= bitx;
            OS_EXIT_CRITICAL();
            OSSched();
        } else
            OS_EXIT_CRITICAL();
    }
    return (OS_NO_ERR);
}
/*
************************************************************
*                     POST TO A PIPE                       *
************************************************************
*/
U08 OS_PipePost(OS_P *pp, U08 OS_HUGE *msg, U16 lenght, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_DP OS_HUGE *dp, *cp;
    U16            i;
    U08            x, y, bitx, bity, p, fast;

    OS_ENTER_CRITICAL();
    if ((pp->OSPSize | pp->OSPDeep) != 0) {
        if (lenght > (pp->OSPSize - sizeof(U16))) {
            OS_EXIT_CRITICAL();
            return (OS_P_LEN_ERR);
        }
        if (pp->OSPEntries >= pp->OSPDeep) {
            if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
                OS_EXIT_CRITICAL();
                return (OS_P_FULL);
            }
#if OS_TASK_DESTROY_EN
            OSTCBCur->OSTCBipc   = pp;
#endif
            OSTCBCur->OSTCBStat |= OS_STAT_P;
            OSTCBCur->OSTCBDly   = timeout;
            y                    = OSTCBCur->OSTCBPrio >> 3;
            x                    = OSTCBCur->OSTCBPrio & 0x07;
            bity                 = OSMapTbl[y];
            bitx                 = OSMapTbl[x];
            if ((U08)(OSRdyTbl[y] &= ~(bitx)) == 0)
                OSRdyGrp &= ~(bity);
            pp->OSPBTbl[y] |= bitx;
            pp->OSPBGrp    |= bity;
            OS_EXIT_CRITICAL();
            OSSched();
            OS_ENTER_CRITICAL();
            if (OSTCBCur->OSTCBStat & OS_STAT_P) {
                if ((U08)(pp->OSPBTbl[y] &= ~(bitx)) == 0)
                    pp->OSPBGrp &= ~(bity);
                OSTCBCur->OSTCBStat = OS_STAT_RDY;
                OS_EXIT_CRITICAL();
                return (OS_TIMEOUT);
            }
            if (pp->OSPEntries >= pp->OSPDeep) {
                OS_EXIT_CRITICAL();
                return (OS_P_FULL);
            }
        }
    }
    fast = 0;
    if ((OSIntNesting == 0) && (OSLockNesting == 0) && (pp->OSPEntries == 0) && (pp->OSPFGrp != 0)) {
        y    = OSUnMapTbl[pp->OSPFGrp];
        x    = OSUnMapTbl[pp->OSPFTbl[y]];
        if ((OSTCBCur->OSTCBPrio > (y << 3) + x) && (pp->OSPData1 == (U08 OS_HUGE *)NULL)) {
            pp->OSPData1   = msg;
            pp->OSPLenght1 = lenght;
            fast           = 1;
        }
    }
    if (fast == 0) {
        if ((pp->OSPSize | pp->OSPDeep) == 0) {
            OS_EXIT_CRITICAL();
            x = OSMemAlloc((void OS_HUGE *)&dp, sizeof(OS_DP) + lenght, 0);
            if (x != OS_NO_ERR) return (x);
            for (i = 0; i < lenght; i++)
                dp->OSDPData[i] = msg[i];
            dp->OSDPLenght = lenght;
            dp->OSDPNext   = NULL;
            OS_ENTER_CRITICAL();
            if (pp->OSPData != NULL) {
                cp = (OS_DP OS_HUGE *)pp->OSPData;
                while (cp->OSDPNext != NULL)
                    cp = cp->OSDPNext;
                cp->OSDPNext = dp;
            } else pp->OSPData = (U08 OS_HUGE *)dp;
        } else {
            y = pp->OSPIn;
            pp->OSPIn++;
            if (pp->OSPIn == pp->OSPDeep)
                pp->OSPIn = 0;
            OSLockNesting++;
            OS_EXIT_CRITICAL();
            pp->OSPData[y * pp->OSPSize]     = (U08)(lenght%256);
            pp->OSPData[y * pp->OSPSize + 1] = (U08)(lenght/256);
            for (i = 0; i < lenght; i++)
                pp->OSPData[y * pp->OSPSize + sizeof(U16) + i] = msg[i];
            OS_ENTER_CRITICAL();
            if (OSLockNesting != 0)
                OSLockNesting--;
        }
    }
    pp->OSPEntries++;
    if (pp->OSPFGrp != 0) {
        y    = OSUnMapTbl[pp->OSPFGrp];
        x    = OSUnMapTbl[pp->OSPFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pp->OSPFTbl[y] &= ~(bitx)) == 0)
            pp->OSPFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_P);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 POST TO FONT OF A PIPE                   *
************************************************************
*/
U08 OS_PipeFrontPost(OS_P *pp, U08 OS_HUGE *msg, U16 lenght, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_DP OS_HUGE *dp;
    U16            i;
    U08            x, y, bitx, bity, p, fast;

    OS_ENTER_CRITICAL();
    if ((pp->OSPSize | pp->OSPDeep) != 0) {
        if (lenght > (pp->OSPSize - sizeof(U16))) {
            OS_EXIT_CRITICAL();
            return (OS_P_LEN_ERR);
        }
        if (pp->OSPEntries >= pp->OSPDeep) {
            if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
                OS_EXIT_CRITICAL();
                return (OS_P_FULL);
            }
#if OS_TASK_DESTROY_EN
            OSTCBCur->OSTCBipc   = pp;
#endif
            OSTCBCur->OSTCBStat |= OS_STAT_P;
            OSTCBCur->OSTCBDly   = timeout;
            y                    = OSTCBCur->OSTCBPrio >> 3;
            x                    = OSTCBCur->OSTCBPrio & 0x07;
            bity                 = OSMapTbl[y];
            bitx                 = OSMapTbl[x];
            if ((U08)(OSRdyTbl[y] &= ~(bitx)) == 0)
                OSRdyGrp &= ~(bity);
            pp->OSPBTbl[y] |= bitx;
            pp->OSPBGrp    |= bity;
            OS_EXIT_CRITICAL();
            OSSched();
            OS_ENTER_CRITICAL();
            if (OSTCBCur->OSTCBStat & OS_STAT_P) {
                if ((U08)(pp->OSPBTbl[y] &= ~(bitx)) == 0)
                    pp->OSPBGrp &= ~(bity);
                OSTCBCur->OSTCBStat = OS_STAT_RDY;
                OS_EXIT_CRITICAL();
                return (OS_TIMEOUT);
            }
            if (pp->OSPEntries >= pp->OSPDeep) {
                OS_EXIT_CRITICAL();
                return (OS_P_FULL);
            }
        }
    }
    fast = 0;
    if ((OSIntNesting == 0) && (OSLockNesting == 0) && (pp->OSPEntries == 0) && (pp->OSPFGrp != 0)) {
        y    = OSUnMapTbl[pp->OSPFGrp];
        x    = OSUnMapTbl[pp->OSPFTbl[y]];
        if ((OSTCBCur->OSTCBPrio > (y << 3) + x) && (pp->OSPData1 == (U08 OS_HUGE *)NULL)) {
            pp->OSPData1   = msg;
            pp->OSPLenght1 = lenght;
            fast           = 1;
        }
    }
    if (fast == 0) {
        if ((pp->OSPSize | pp->OSPDeep) == 0) {
            OS_EXIT_CRITICAL();
            x = OSMemAlloc((void OS_HUGE *)&dp, sizeof(OS_DP) + lenght, 0);
            if (x != OS_NO_ERR) return (x);
            for (i = 0; i < lenght; i++)
                dp->OSDPData[i] = msg[i];
            dp->OSDPLenght = lenght;
            OS_ENTER_CRITICAL();
            dp->OSDPNext = (OS_DP OS_HUGE *)pp->OSPData;
            pp->OSPData  = (U08 OS_HUGE *)dp;
        } else {
            pp->OSPOut--;
            if (pp->OSPOut >= pp->OSPDeep)
                pp->OSPOut = pp->OSPDeep - 1;
            y = pp->OSPOut;
            OSLockNesting++;
            OS_EXIT_CRITICAL();
            pp->OSPData[y * pp->OSPSize]     = (U08)(lenght%256);
            pp->OSPData[y * pp->OSPSize + 1] = (U08)(lenght/256);
            for (i = 0; i < lenght; i++)
                pp->OSPData[y * pp->OSPSize + sizeof(U16) + i] = msg[i];
            OS_ENTER_CRITICAL();
            if (OSLockNesting != 0)
                OSLockNesting--;
        }
    }
    pp->OSPEntries++;
    if (pp->OSPFGrp != 0) {
        y    = OSUnMapTbl[pp->OSPFGrp];
        x    = OSUnMapTbl[pp->OSPFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pp->OSPFTbl[y] &= ~(bitx)) == 0)
            pp->OSPFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_P);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*             ABBORT HIGHEST PENDER ON A PIPE              *
************************************************************
*/
U08 OS_PipePendAbbort(OS_P *pp) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pp->OSPFGrp != 0) {
        y    = OSUnMapTbl[pp->OSPFGrp];
        x    = OSUnMapTbl[pp->OSPFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pp->OSPFTbl[y] &= ~(bitx)) == 0)
            pp->OSPFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_P);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}
/*
************************************************************
*             ABBORT HIGHEST POSTER ON A PIPE              *
************************************************************
*/
U08 OS_PipePostAbbort(OS_P *pp) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pp->OSPBGrp != 0) {
        y    = OSUnMapTbl[pp->OSPBGrp];
        x    = OSUnMapTbl[pp->OSPBTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pp->OSPBTbl[y] &= ~(bitx)) == 0)
            pp->OSPBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_P);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*                 DELETE A TASK FROM A PIPE                *
************************************************************
*/
void OSPipeDestroyTask(OS_P *pp, U08 prio)
{
    U08          x, y, bitx, bity;

    y      = prio >> 3;
    x      = prio & 0x07;
    bity   = OSMapTbl[y];
    bitx   = OSMapTbl[x];
    if ((U08)(pp->OSPFTbl[y] &= ~(bitx)) == 0)
        pp->OSPFGrp &= ~(bity);
    if ((U08)(pp->OSPBTbl[y] &= ~(bitx)) == 0)
        pp->OSPBGrp &= ~(bity);
    OSTCBPrioTbl[prio]->OSTCBDly   =  0;
    OSTCBPrioTbl[prio]->OSTCBStat &= ~(OS_STAT_P);
    OSRdyGrp                      |=  bity;
    OSRdyTbl[y]                   |=  bitx;
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_P_EN */

/************************* END ****************************/


