/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_smq.c                         *
*              SMART MESSAGE QUEUE SERVICE                 *
***********************************************************/

#if OS_SMQ_EN
/*
************************************************************
*              CHANGE MEMORY OWNER (internal)              *
************************************************************
*/
U08 OSSMQueueMemoryOwner(void OS_HUGE *p, U08 TCBid)
{
    OS_MEM OS_HUGE *cm;

    cm = (OS_MEM OS_HUGE *)((U08 OS_HUGE *)(p) - sizeof(OS_MEM));
    if ((OS_MemVerifyPtr(p) == 0) && (OSMemoryCheck(cm) == 0)) {
        cm->OSMId  = TCBid;
        return 0;
    }
    return 1;
}
/*
************************************************************
*                 INITIALIZE MESSAGE QUEUE                 *
************************************************************
*/
U08 OS_SMQueueInit(OS_SMQ *psmq, void OS_HUGE *Sdata, U16 deep) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08 i;

    OS_ENTER_CRITICAL();
    if (psmq) {
        psmq->OSSData     = Sdata;
        psmq->OSSIn       = 0x00;
        psmq->OSSOut      = 0x00;
        psmq->OSSDeep     = deep;
        psmq->OSSEntries  = 0;
        psmq->OSSFGrp     = 0;
        psmq->OSSBGrp     = 0;
        for (i = 0; i < OS_PRIO_TBL; i++) {
            psmq->OSSFTbl[i]  = 0;
            psmq->OSSBTbl[i]  = 0;
        }
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_SMQ_ERR);
    }
}
/*
************************************************************
*                 GET INFO OF MESSAGE QUEUE                *
************************************************************
*/
U08 OS_SMQueueInfo(OS_SMQ *psmq, U16 *deep, U16 *used, U08 *prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y;

    OS_ENTER_CRITICAL();
    *deep = psmq->OSSDeep;
    *used = psmq->OSSEntries;
    if (psmq->OSSFGrp != 0) {
        y    = OSUnMapTbl[psmq->OSSFGrp];
        x    = OSUnMapTbl[psmq->OSSFTbl[y]];
       *prio = (y << 3) + x;
    } else
       *prio = 0;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*             CLEAR ALL DATA OF MESSAGE QUEUE              *
************************************************************
*/
U08 OS_SMQueueClear(OS_SMQ *psmq) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    while(psmq->OSSIn != psmq->OSSOut) {
        if (OSSMQueueMemoryOwner(psmq->OSSData[psmq->OSSOut], 0) == 0) {
            OSLockNesting++;
            OS_EXIT_CRITICAL();
            OS_MemFree(psmq->OSSData[psmq->OSSOut]);
            OS_ENTER_CRITICAL();
            if (OSLockNesting != 0)
                OSLockNesting--;
        }
        psmq->OSSOut++;
        if (psmq->OSSOut == psmq->OSSDeep)
            psmq->OSSOut = 0;
    }
    psmq->OSSIn       = 0;
    psmq->OSSOut      = 0;
    psmq->OSSEntries  = 0;
    if (psmq->OSSBGrp != 0) {
        y                = OSUnMapTbl[psmq->OSSBGrp];
        x                = OSUnMapTbl[psmq->OSSBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(psmq->OSSBTbl[y] &= ~(bitx)) == 0)
            psmq->OSSBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SMQ);
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 POST TO A MESSAGE QUEUE                  *
************************************************************
*/
U08 OS_SMQueuePost(OS_SMQ *psmq, void OS_HUGE **msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psmq->OSSEntries >= psmq->OSSDeep) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_SMQ_FULL);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = psmq;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_SMQ;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        psmq->OSSBTbl[y] |= bitx;
        psmq->OSSBGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if (OSTCBCur->OSTCBStat & OS_STAT_SMQ) {
            if ((U08)(psmq->OSSBTbl[y] &= ~(bitx)) == 0)
                psmq->OSSBGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (psmq->OSSEntries >= psmq->OSSDeep) {
            OS_EXIT_CRITICAL();
            return (OS_SMQ_FULL);
        }
    }
    psmq->OSSData[psmq->OSSIn++] = *msg;
    if (OSSMQueueMemoryOwner(*msg, 0) == 0)
        *msg = NULL;
    psmq->OSSEntries++;
    if (psmq->OSSIn == psmq->OSSDeep)
        psmq->OSSIn = 0;
    if (psmq->OSSFGrp != 0) {
        y    = OSUnMapTbl[psmq->OSSFGrp];
        x    = OSUnMapTbl[psmq->OSSFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(psmq->OSSFTbl[y] &= ~(bitx)) == 0)
            psmq->OSSFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SMQ);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*            POST TO FONT OF A MESSAGE QUEUE               *
************************************************************
*/
U08 OS_SMQueueFrontPost(OS_SMQ * psmq, void OS_HUGE **msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psmq->OSSEntries >= psmq->OSSDeep) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_SMQ_FULL);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = psmq;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_SMQ;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        psmq->OSSBTbl[y] |= bitx;
        psmq->OSSBGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if (OSTCBCur->OSTCBStat & OS_STAT_SMQ) {
            if ((U08)(psmq->OSSBTbl[y] &= ~(bitx)) == 0)
                psmq->OSSBGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (psmq->OSSEntries >= psmq->OSSDeep) {
            OS_EXIT_CRITICAL();
            return (OS_SMQ_FULL);
        }
    }
    psmq->OSSOut--;
    if (psmq->OSSOut >= psmq->OSSDeep)
        psmq->OSSOut = psmq->OSSDeep-1;
    psmq->OSSData[psmq->OSSOut] = *msg;
    if (OSSMQueueMemoryOwner(*msg, 0) == 0)
        *msg = NULL;
    psmq->OSSEntries++;
    if (psmq->OSSFGrp != 0) {
        y    = OSUnMapTbl[psmq->OSSFGrp];
        x    = OSUnMapTbl[psmq->OSSFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(psmq->OSSFTbl[y] &= ~(bitx)) == 0)
            psmq->OSSFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SMQ);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 PEND ON A MESSAGE QUEUE                  *
************************************************************
*/
U08 OS_SMQueuePend(OS_SMQ * psmq, void OS_HUGE **msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psmq->OSSEntries == 0) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_SMQ_NODATA);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = psmq;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_SMQ;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        psmq->OSSFTbl[y] |= bitx;
        psmq->OSSFGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if ((OSTCBCur->OSTCBStat & OS_STAT_SMQ) && (psmq->OSSEntries == 0)){
            if ((U08)(psmq->OSSFTbl[y] &= ~(bitx)) == 0)
                psmq->OSSFGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            *msg                = NULL;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (psmq->OSSEntries == 0) {
            OS_EXIT_CRITICAL();
            return (OS_SMQ_NODATA);
        }
    }
    *msg = psmq->OSSData[psmq->OSSOut++];
    OSSMQueueMemoryOwner(*msg, OSTCBCur->OSTCBId);
    psmq->OSSEntries--;
    if (psmq->OSSOut == psmq->OSSDeep)
        psmq->OSSOut = 0;
    if (psmq->OSSBGrp != 0) {
        y                = OSUnMapTbl[psmq->OSSBGrp];
        x                = OSUnMapTbl[psmq->OSSBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(psmq->OSSBTbl[y] &= ~(bitx)) == 0)
            psmq->OSSBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SMQ);
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*        ABBORT HIGHEST PENDER ON A MESSAGE QUEUE          *
************************************************************
*/
U08 OS_SMQueuePendAbbort(OS_SMQ * psmq) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psmq->OSSFGrp != 0) {
        y    = OSUnMapTbl[psmq->OSSFGrp];
        x    = OSUnMapTbl[psmq->OSSFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(psmq->OSSFTbl[y] &= ~(bitx)) == 0)
            psmq->OSSFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SMQ);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}
/*
************************************************************
*        ABBORT HIGHEST POSTER ON A MESSAGE QUEUE          *
************************************************************
*/
U08 OS_SMQueuePostAbbort(OS_SMQ * psmq) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psmq->OSSBGrp != 0) {
        y    = OSUnMapTbl[psmq->OSSBGrp];
        x    = OSUnMapTbl[psmq->OSSBTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(psmq->OSSBTbl[y] &= ~(bitx)) == 0)
            psmq->OSSBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SMQ);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*            DELETE A TASK FROM A MESSAGE QUEUE            *
************************************************************
*/
void OSSMQueueDestroyTask(OS_SMQ *psmq, U08 prio)
{
    U08          x, y, bitx, bity;

    y      = prio >> 3;
    x      = prio & 0x07;
    bity   = OSMapTbl[y];
    bitx   = OSMapTbl[x];
    if ((U08)(psmq->OSSFTbl[y] &= ~(bitx)) == 0)
        psmq->OSSFGrp &= ~(bity);
    if ((U08)(psmq->OSSBTbl[y] &= ~(bitx)) == 0)
        psmq->OSSBGrp &= ~(bity);
    OSTCBPrioTbl[prio]->OSTCBDly   =  0;
    OSTCBPrioTbl[prio]->OSTCBStat &= ~(OS_STAT_SMQ);
    OSRdyGrp                      |=  bity;
    OSRdyTbl[y]                   |=  bitx;
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_SMQ_EN */

/************************* END ****************************/



