/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_task.c                        *
*                      TASK SERVICE                        *
***********************************************************/

/*
************************************************************
*                    TABLES IN CODE                        *
************************************************************
*/
U08 OS_CONST OS_ROMDATA OSMapTbl[]   = {
    0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80};

U08 OS_CONST OS_ROMDATA OSUnMapTbl[] = {
    0, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    7, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
    4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0};
/*
************************************************************
*                GLOBAL AND LOCAL VARIABLES                *
************************************************************
*/
    OS_STK_TYPE     OSIdleTaskStk[OS_IDLE_STK_SIZE];
    OS_TCB          OSTCBTbl[OS_MAX_TASKS];
    OS_TCB         *OSTCBCur;
    OS_TCB         *OSTCBHighRdy;
    OS_TCB         *OSTCBList;
    OS_TCB         *OSTCBPrioTbl[OS_MIN_PRIO];
    BOOLEAN         OSRunning;
    U08             OSRdyGrp;
    U08             OSRdyTbl[OS_PRIO_TBL];
    U16             OSLockNesting;
    U08             OSIntNesting;
    OS_TCB         *OSTCBFreeList;
    U32             OSSystemCnt;
    U16             OSIdleCnt;
/*
************************************************************
*                LOCAL FUNCTION PROTOTYPES                 *
************************************************************
*/
void OS_FAR OSTaskIdle(void *Tdata) KC51_REENTRANT;
#if  OS_TMR_EN
 void OS_FAR OSTaskTimer(void *Tdata) KC51_REENTRANT;
#endif
/*
************************************************************
*                   pC/OS INITIALIZATION                   *
************************************************************
*/
void OS_Init(void) KC51_REENTRANT
{
    U08 i;

    OSTCBCur       = (OS_TCB *)NULL;
    OSTCBList      = (OS_TCB *)NULL;
    OSSystemCnt    = 0;
    OSIdleCnt      = 0;
    OSIntNesting   = 0;
    OSLockNesting  = 0;
    OSRunning      = 0;
    OSRdyGrp       = 0;
    for (i = 0; i < OS_PRIO_TBL; i++)
        OSRdyTbl[i] = 0;
    for (i = 0; i < OS_MIN_PRIO; i++)
        OSTCBPrioTbl[i] = (OS_TCB *)NULL;
    for (i = 0; i < (OS_MAX_TASKS - 1); i++) {
        OSTCBTbl[i].OSTCBId   = (OS_STK_TYPE)i;
        OSTCBTbl[i].OSTCBPrio = 0xFF;
        OSTCBTbl[i].OSTCBNext = &OSTCBTbl[i+1];
    }
    OSTCBTbl[OS_MAX_TASKS-1].OSTCBId   = (U08)(OS_MAX_TASKS-1);
    OSTCBTbl[OS_MAX_TASKS-1].OSTCBNext = (OS_TCB *)NULL;
    OSTCBFreeList                      = &OSTCBTbl[0];
#if OS_TMR_EN
    OSTimerCnt     = 0;
    TMR_FirstPtr   = NULL;
    OS_SemInit(&OSTimerSignal, 0);
#endif
    OS_TaskCreate(OSTaskIdle,  (void *)NULL, OSIdleTaskStk,  OS_IDLE_STK_SIZE, OS_LO_PRIO);
#if OS_TMR_EN
    OS_TaskCreate(OSTaskTimer, (void *)NULL, OSTimerTaskStk, OS_TMR_STK_SIZE,  OS_TMR_PRIO);
#endif
}
/*
************************************************************
*                       IDLE TASK                          *
************************************************************
*/
void OS_FAR OSTaskIdle(void *Tdata)  KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    Tdata = Tdata;
    while (1) {
        OS_ENTER_CRITICAL();
        OSIdleCnt++;
        OS_EXIT_CRITICAL();
    }
}
/*
************************************************************
*                    START MULTITASKING                    *
************************************************************
*/
void OS_Start(void) KC51_REENTRANT
{
    U08          x, y, p;

    y            = OSUnMapTbl[OSRdyGrp];
    x            = OSRdyTbl[y];
    p            = (y << 3) + OSUnMapTbl[x];
    OSTCBHighRdy = OSTCBPrioTbl[p];
    OSTCBCur     = OSTCBHighRdy;
    OSRunning    = 1;
    OSStartHighRdy();
}
/*
************************************************************
*                    CREATE A TASK                         *
************************************************************
*/
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U16 stksize, U08 p) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_STK_TYPE *stk;
    U08          err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk = OSTaskInitStk(task, Tdata, &pstk, stksize);
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstk);
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}
/*
************************************************************
*                RUN HIGHEST PRIORITY TASK                 *
************************************************************
*/
void OSSched(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, p;

    OS_ENTER_CRITICAL();
    if ((OSLockNesting == 0) && (OSIntNesting == 0)) {
        y            = OSUnMapTbl[OSRdyGrp];
        x            = OSRdyTbl[y];
        p            = (y << 3) + OSUnMapTbl[x];
        OSTCBHighRdy = OSTCBPrioTbl[p];
#if OS_STK_CHECK_EN
 #ifndef OS_STK_CHECK_SW_off
        if(*(OS_STK_TYPE *)OSTCBCur->OSTCBStkEnd != OSTCBCur->OSTCBId)
            OSStkEndError();
 #endif
#endif
        if ((OSRunning == 1) && (OSTCBHighRdy != OSTCBCur))
            OS_TASK_SW();
    }
    OS_EXIT_CRITICAL();
}
/*
************************************************************
*                        ENTER ISR                         *
************************************************************
*/
void OSIntEnter(void)
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    OSIntNesting++;
    OS_EXIT_CRITICAL();
}
/*
************************************************************
*                        EXIT ISR                          *
************************************************************
*/
void OSIntExit(void)
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, p, f;

    OS_ENTER_CRITICAL();
    if (OSIntNesting != 0)
        OSIntNesting--;
    if ((OSIntNesting == 0) && ((OSLockNesting == 0) || (OSTCBHighRdy == NULL))) {
        if (OSTCBHighRdy == NULL) {
            f = 1; OSLockNesting = 0;
        } else f = 0;
        y            = OSUnMapTbl[OSRdyGrp];
        x            = OSRdyTbl[y];
        p            = (y << 3) + OSUnMapTbl[x];
        OSTCBHighRdy = OSTCBPrioTbl[p];
#if OS_STK_CHECK_EN
 #ifndef OS_STK_CHECK_SW_off
        if(*(OS_STK_TYPE *)OSTCBCur->OSTCBStkEnd != OSTCBCur->OSTCBId)
            OSStkEndError();
 #endif
#endif
        if ((OSRunning == 1) && ((OSTCBHighRdy != OSTCBCur) || (f != 0)))
           OSIntCtxSw();
    }
    OS_EXIT_CRITICAL();
}
/*
************************************************************
*                    INITIALIZE TCB                        *
************************************************************
*/
U08 OSTCBInit(U08 p, void OS_FAR *pstk, void OS_FAR *pstkend) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_TCB      *ptr;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    ptr  = OSTCBGetFree();
    if (ptr == (OS_TCB *)NULL) {
        OS_EXIT_CRITICAL();
        return (OS_TASK_NOT_EXIST);
    }
    ptr->OSTCBPrio   = (U08)p;
    ptr->OSTCBStat   = OS_STAT_RDY;
    ptr->OSTCBDly    = 0;
#if OS_TASK_DESTROY_EN
    ptr->OSTCBipc    = NULL;
 #if OS_MUX_EN
    ptr->OSTCBmux    = NULL;
 #endif
#endif
    ptr->OSTCBStkPtr = pstk;
    OSTCBPrioTbl[p]  = ptr;
    ptr->OSTCBNext   = OSTCBList;
    ptr->OSTCBPrev   = (OS_TCB *)NULL;
    if (OSTCBList != (OS_TCB *)NULL)
        OSTCBList->OSTCBPrev = ptr;
    OSTCBList        = ptr;
#if OS_STK_CHECK_EN
 #if OS_STK_CHECK_FILL
    { 
        OS_STK_TYPE *stk  = (pstk < pstkend)? (OS_STK_TYPE *)pstkend : (OS_STK_TYPE *)pstk; 
        OS_STK_TYPE *estk = (pstk < pstkend)? (OS_STK_TYPE *)pstk : (OS_STK_TYPE *)pstkend; 
        while(stk > estk) *--stk = (OS_STK_TYPE)0xEFEFEFEF; 
    } 
 #endif
    ptr->OSTCBStkEnd = (pstk < pstkend)?
        (void OS_FAR *)((OS_STK_TYPE)pstkend - (OS_STK_RESERVE * sizeof(OS_STK_TYPE))) :
        (void OS_FAR *)((OS_STK_TYPE)pstkend + (OS_STK_RESERVE * sizeof(OS_STK_TYPE)));
    *(OS_STK_TYPE *)ptr->OSTCBStkEnd = (OS_STK_TYPE)ptr->OSTCBId;
#endif
    OSRdyGrp        |= OSMapTbl[p >> 3];
    OSRdyTbl[p >> 3]|= OSMapTbl[p & 0x07];
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                  PROCESS SYSTEM TICK                     *
************************************************************
*/
void OSSystemTick(void)
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          p;
    OS_TCB      *ptcb;

    OS_ENTER_CRITICAL();
    OSSystemCnt++;
    if (OSRunning == 0) {
        OS_EXIT_CRITICAL();
        return;
    }
    OS_EXIT_CRITICAL();
    ptcb = OSTCBList;
    while ((ptcb) && (ptcb->OSTCBPrio != OS_LO_PRIO)) {
        OS_Lock();
        if (ptcb->OSTCBDly != 0) {
            if ((--ptcb->OSTCBDly) == 0) {
                if (!(ptcb->OSTCBStat & OS_STAT_SUSP)) {
                    p                 = ptcb->OSTCBPrio;
                    OSRdyGrp         |= OSMapTbl[p >> 3];
                    OSRdyTbl[p >> 3] |= OSMapTbl[p & 0x07];
                } else
                    ptcb->OSTCBDly = 1;
            }
        }
        ptcb = ptcb->OSTCBNext;
        OS_Unlock();
    }
}
/*
************************************************************
*                   PREVENT SCHEDULING                     *
************************************************************
*/
void OS_Lock(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    OSLockNesting++;
    OS_EXIT_CRITICAL();
}
/*
************************************************************
*                    ENABLE SCHEDULING                     *
************************************************************
*/
void OS_Unlock(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    OS_EXIT_CRITICAL();
    if (OSRunning == 1)
        OSSched();
}
/*
************************************************************
*                GET TCB FROM FREE TCB LIST                *
************************************************************
*/
OS_TCB *OSTCBGetFree(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_TCB      *ptcb;

    OS_ENTER_CRITICAL();
    ptcb          = OSTCBFreeList;
    if (ptcb != (OS_TCB *)NULL)
        OSTCBFreeList = ptcb->OSTCBNext;
    OS_EXIT_CRITICAL();
    return (ptcb);
}

#if OS_TASK_EXT_EN
/*
************************************************************
*    DELAY CURRENT TASK 'n' TICKS   (n from 1 to 65535)    *
************************************************************
*/
void OS_TimeDly(U16 ticks) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          p;

    OS_ENTER_CRITICAL();
    if (OSIntNesting > 0) {
        OS_EXIT_CRITICAL();
        return;
    }
    p = OSTCBCur->OSTCBPrio;
    if ((OSRdyTbl[p >> 3] &= ~(OSMapTbl[p & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[p >> 3]);
    OSTCBCur->OSTCBDly = ticks;
    OS_EXIT_CRITICAL();
    OSSched();
}
/*
************************************************************
*                RESUME A DELAYED TASK BY PRIO             *
************************************************************
*/
U08 OS_TimeDlyResume(U08 prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_TCB      *ptcb;

    if (prio >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    ptcb = (OS_TCB *)OSTCBPrioTbl[prio];
    if (ptcb != (OS_TCB *)NULL) {
        if (ptcb->OSTCBDly != 0) {
            ptcb->OSTCBDly  = 0;
            if (!(ptcb->OSTCBStat & OS_STAT_SUSP)) {
                OSRdyGrp            |= OSMapTbl[prio >> 3];
                OSRdyTbl[prio >>3]  |= OSMapTbl[prio & 0x07];
                OS_EXIT_CRITICAL();
                OSSched();
            } else
                OS_EXIT_CRITICAL();
            return (OS_NO_ERR);
        }
        OS_EXIT_CRITICAL();
        return (OS_TIME_NOT_DLY);
    }
    OS_EXIT_CRITICAL();
    return (OS_TASK_NOT_EXIST);
}
/*
************************************************************
*             RESUME A DELAYED TASK BY UNIQUE ID           *
************************************************************
*/
U08 OS_TimeDlyIdResume(U08 id) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          prio;

    if (id >= OS_MAX_TASKS)
        return (OS_TASK_NOT_EXIST);
    OS_ENTER_CRITICAL();
    prio = OS_TaskGetPrio(id);
    if(prio != 0xFF) {
        prio = OS_TimeDlyResume(prio);
        OS_EXIT_CRITICAL();
        return (prio);
    }
    OS_EXIT_CRITICAL();
    return (OS_TASK_NOT_EXIST);
}
/*
************************************************************
*                    SET SYSTEM CLOCK                      *
************************************************************
*/
void OS_TimeSet(U32 ticks) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    OSSystemCnt = ticks;
    OS_EXIT_CRITICAL();
}
/*
************************************************************
*                  GET CURRENT SYSTEM TIME                 *
************************************************************
*/
U32 OS_TimeGet(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32          ticks;

    OS_ENTER_CRITICAL();
    ticks = OSSystemCnt;
    OS_EXIT_CRITICAL();
    return (ticks);
}
/*
************************************************************
*              CHANGE PRIORITY OF RUNNING TASK             *
************************************************************
*/
U08 OS_ChangePrio(U08 newp) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          oldp;

    if (newp >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[newp] != (OS_TCB *)NULL) {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
    oldp = OSTCBCur->OSTCBPrio;
#if OS_MUX_EN
    if (OSTCBPrioTbl[oldp] == (OS_TCB *)1) {
        OS_EXIT_CRITICAL();
        return (OS_MUX_USED);
    }
#endif
    if ((OSRdyTbl[oldp >> 3] &= ~(OSMapTbl[oldp & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[oldp >> 3]);
    OSRdyGrp            |= OSMapTbl[newp >> 3];
    OSRdyTbl[newp >> 3] |= OSMapTbl[newp & 0x07];
    OSTCBCur->OSTCBPrio  = newp;
    OSTCBPrioTbl[newp]   = OSTCBCur;
    OSTCBPrioTbl[oldp]   = (OS_TCB *)NULL;
    OS_EXIT_CRITICAL();
    OSSched();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 CHANGE PRIORITY OF A TASK                *
************************************************************
*/
U08 OS_TaskChangePrio(U08 oldp, U08 newp) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    if ((oldp >= OS_MIN_PRIO) && (oldp != OS_PRIO_SELF))
        return (OS_PRIO_INVALID);
    if (newp >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[newp] != (OS_TCB *)NULL) {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
    if (oldp == OS_PRIO_SELF)
        oldp = OSTCBCur->OSTCBPrio;
    else {
        if (oldp != OSTCBCur->OSTCBPrio) {
            if (OSTCBPrioTbl[oldp]->OSTCBStat != OS_STAT_RDY) {
                OS_EXIT_CRITICAL();
                return (OS_TASK_NOT_RDY);
            }
        }
    }
#if OS_MUX_EN
    if (OSTCBPrioTbl[oldp] == (OS_TCB *)1) {
        OS_EXIT_CRITICAL();
        return (OS_MUX_USED);
    }
#endif
    if ((OSRdyTbl[oldp >> 3] &= ~(OSMapTbl[oldp & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[oldp >> 3]);
    OSRdyGrp            |= OSMapTbl[newp >> 3];
    OSRdyTbl[newp >> 3] |= OSMapTbl[newp & 0x07];
    OSTCBCur->OSTCBPrio  = newp;
    OSTCBPrioTbl[newp]   = OSTCBCur;
    OSTCBPrioTbl[oldp]   = (OS_TCB *)NULL;
    OS_EXIT_CRITICAL();
    OSSched();
    return (OS_NO_ERR);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*                  DESTROY A TASK (internal)               *
************************************************************
*/
U08 OSTaskDestroy(U08 prio) KC51_REENTRANT
{
    if (prio != OSTCBCur->OSTCBPrio) {
        if (OSTCBPrioTbl[prio]->OSTCBStat != OS_STAT_RDY) {
            switch (OSTCBPrioTbl[prio]->OSTCBStat & ~OS_STAT_POST) {
 #if OS_MBOX_EN
            case OS_STAT_MBOX:
                OSMboxDestroyTask((OS_MBOX *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
 #if OS_SMQ_EN
            case OS_STAT_SMQ:
                OSSMQueueDestroyTask((OS_SMQ *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
 #if OS_SEM_EN
            case OS_STAT_SEM:
                OSSemDestroyTask((OS_SEM *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
 #if OS_Q_EN
            case OS_STAT_Q:
                OSQueueDestroyTask((OS_Q *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
 #if OS_P_EN
            case OS_STAT_P:
                OSPipeDestroyTask((OS_P *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
 #if OS_EVG_EN
            case OS_STAT_EVG:
                OSEvgDestroyTask((OS_EVG *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
 #if OS_MUX_EN
            case OS_STAT_MUX:
                OSMutexDestroyTask((OS_MUX *)OSTCBPrioTbl[prio]->OSTCBipc, prio);
                break;
 #endif
            default:
                return (1);
//                break;
            }
            OSTCBPrioTbl[prio]->OSTCBipc = 0;
        }
 #if OS_MUX_EN
        if (OSTCBPrioTbl[prio]->OSTCBmux != NULL) {
                prio = OSMutexFreeTask(OSTCBPrioTbl[prio]->OSTCBmux, prio);
                if (prio == 0xFF) return (1);
        }
 #endif
    }
    if ((OSRdyTbl[prio >> 3] &= ~(OSMapTbl[prio & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[prio >> 3]);
    if (OSTCBPrioTbl[prio]->OSTCBPrev           == (OS_TCB *)NULL) {
        OSTCBPrioTbl[prio]->OSTCBNext->OSTCBPrev = (OS_TCB *)NULL;
        OSTCBList                                =  OSTCBPrioTbl[prio]->OSTCBNext;
    } else {
        OSTCBPrioTbl[prio]->OSTCBPrev->OSTCBNext = OSTCBPrioTbl[prio]->OSTCBNext;
        OSTCBPrioTbl[prio]->OSTCBNext->OSTCBPrev = OSTCBPrioTbl[prio]->OSTCBPrev;
    }
    OSTCBPrioTbl[prio]->OSTCBNext = OSTCBFreeList;
    OSTCBFreeList                 = OSTCBPrioTbl[prio];
    OSTCBPrioTbl[prio]            = (OS_TCB *)NULL;
    return (0);
}
/*
************************************************************
*            DESTROYES A TASK BY IT'S UNIQUE ID            *
************************************************************
*/
U08 OS_TaskIdDestroy(U08 id) KC51_REENTRANT
{
 #if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
 #endif
    U08          prio;

    if (id >= OS_MAX_TASKS)
        return (OS_TASK_NOT_EXIST);
    OS_ENTER_CRITICAL();
    if ((id == 0) || ((OSTCBCur->OSTCBId == id) && (OSIntNesting == 0))) {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_INVALID);
    }
    prio = OSTCBTbl[id].OSTCBPrio;
    if (OSTaskDestroy(prio)) {
        OS_EXIT_CRITICAL();
        return (OS_TASK_NOT_RDY);
    }
 #if OS_MEM_EN
    OSMemCleanUp(id);
 #endif
    if(OSTCBCur->OSTCBId == id)
        OSTCBHighRdy = NULL;
    OS_EXIT_CRITICAL();
    OSSched();
    return (OS_NO_ERR);
}
#endif /* OS_TASK_DESTROY_EN */

/*
************************************************************
*                  DELETE A TASK BY PRIO                   *
************************************************************
*/
U08 OS_TaskDelete(U08 prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    if ((prio >= OS_MIN_PRIO) && (prio != OS_PRIO_SELF))
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (prio == OS_PRIO_SELF)
        prio = OSTCBCur->OSTCBPrio;
#if OS_MUX_EN
    if (OSTCBPrioTbl[prio] == (OS_TCB *)1) {
        OS_EXIT_CRITICAL();
        return (OS_MUX_USED);
    }
#endif
    if (prio != OSTCBCur->OSTCBPrio) {
        if (OSTCBPrioTbl[prio]->OSTCBStat != OS_STAT_RDY) {
            OS_EXIT_CRITICAL();
            return (OS_TASK_NOT_RDY);
        }
    }
    if ((OSRdyTbl[prio >> 3] &= ~(OSMapTbl[prio & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[prio >> 3]);
    if (OSTCBPrioTbl[prio]->OSTCBPrev           == (OS_TCB *)NULL) {
        OSTCBPrioTbl[prio]->OSTCBNext->OSTCBPrev = (OS_TCB *)NULL;
        OSTCBList                                =  OSTCBPrioTbl[prio]->OSTCBNext;
    } else {
        OSTCBPrioTbl[prio]->OSTCBPrev->OSTCBNext = OSTCBPrioTbl[prio]->OSTCBNext;
        OSTCBPrioTbl[prio]->OSTCBNext->OSTCBPrev = OSTCBPrioTbl[prio]->OSTCBPrev;
    }
    OSTCBPrioTbl[prio]->OSTCBNext = OSTCBFreeList;
    OSTCBFreeList                 = OSTCBPrioTbl[prio];
    OSTCBPrioTbl[prio]            = (OS_TCB *)NULL;
    OS_EXIT_CRITICAL();
    OSSched();
    return (OS_NO_ERR);
}
/*
************************************************************
*                   DELETE A TASK BY ID                    *
************************************************************
*/
U08 OS_TaskIdDelete(U08 id) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          prio;

    if ((id >= OS_MAX_TASKS) && (id != OS_ID_SELF))
        return (OS_TASK_NOT_EXIST);
    OS_ENTER_CRITICAL();
    prio = OS_TaskGetPrio(id);
    if(prio != 0xFF) {
        prio = OS_TaskDelete(prio);
        OS_EXIT_CRITICAL();
        return (prio);
    }
    OS_EXIT_CRITICAL();
    return (OS_TASK_NOT_EXIST);
}
/*
************************************************************
*              GET UNIQUE ID OF A TASK BY PRIO             *
************************************************************
*/
U08 OS_TaskGetID(U08 prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          id;

    if ((prio >= OS_MIN_PRIO) && (prio != OS_PRIO_SELF))
        return (0xFF);
    OS_ENTER_CRITICAL();
    if (prio == OS_PRIO_SELF)
        prio = OSTCBCur->OSTCBPrio;
    if (OSTCBPrioTbl[prio] != (OS_TCB *)1)
        id = OSTCBPrioTbl[prio]->OSTCBId;
    else id = 0xFF;
    OS_EXIT_CRITICAL();
    return (id);
}
/*
************************************************************
*                 GET PRIO OF A TASK BY ID                 *
************************************************************
*/
U08 OS_TaskGetPrio(U08 id) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          prio;

    if ((id >= OS_MAX_TASKS) && (id != OS_ID_SELF))
        return (0xFF);
    OS_ENTER_CRITICAL();
    if (id == OS_PRIO_SELF)
        id = OSTCBCur->OSTCBId;
    prio = OSTCBTbl[id].OSTCBPrio;
    OS_EXIT_CRITICAL();
    return (prio);
}
/*
************************************************************
*              GET STATUS OF A TASK BY PRIO                *
************************************************************
*/
U08 OS_TaskGetStatus(U08 prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          state;

    if (prio >= OS_MIN_PRIO)
        return (0xFF);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[prio] != (OS_TCB *)1)
        state = OSTCBPrioTbl[prio]->OSTCBStat;
    else state = 0xFF;
    OS_EXIT_CRITICAL();
    return (state);
}
/*
************************************************************
*               GET STATUS OF A TASK BY ID                 *
************************************************************
*/
U08 OS_TaskIdGetStatus(U08 id) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          state;

    if (id >= OS_MAX_TASKS)
        return (0xFF);
    OS_ENTER_CRITICAL();
    state = OSTCBTbl[id].OSTCBStat;
    OS_EXIT_CRITICAL();
    return (state);
}
/*
************************************************************
*                 SUSPEND A TASK BY PRIO                   *
************************************************************
*/
U08 OS_TaskSuspend(U08 prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_TCB      *ptcb;
    BOOLEAN      self;

    if (prio == OS_LO_PRIO)
        return (OS_SUSPEND_IDLE);
    if ((prio >= OS_MIN_PRIO) && (prio != OS_PRIO_SELF))
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (prio == OS_PRIO_SELF) {
        prio = OSTCBCur->OSTCBPrio;
        self = TRUE;
    } else {
        if(prio == OSTCBCur->OSTCBPrio)
            self = TRUE;
        else
            self = FALSE;
    }
#if OS_MUX_EN
    if (OSTCBPrioTbl[prio] == (OS_TCB *)1) {
        OS_EXIT_CRITICAL();
        return (OS_MUX_USED);
    }
#endif
    if ((ptcb = OSTCBPrioTbl[prio]) == (OS_TCB *)NULL) {
        OS_EXIT_CRITICAL();
        return (OS_TASK_SUSP_PRIO);
    }
    if ((OSRdyTbl[prio >> 3] &= ~(OSMapTbl[prio & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[prio>>3]);
    ptcb->OSTCBStat |= OS_STAT_SUSP;
    OS_EXIT_CRITICAL();
    if (self == TRUE)
        OSSched();
    return (OS_NO_ERR);
}
/*
************************************************************
*                  SUSPEND A TASK BY ID                    *
************************************************************
*/
U08 OS_TaskIdSuspend(U08 id) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          prio;

    if (id >= OS_MAX_TASKS)
        return (OS_TASK_NOT_EXIST);
    OS_ENTER_CRITICAL();
    prio = OS_TaskGetPrio(id);
    if(prio != 0xFF) {
        prio = OS_TaskSuspend(prio);
        OS_EXIT_CRITICAL();
        return (prio);
    }
    OS_EXIT_CRITICAL();
    return (OS_TASK_NOT_EXIST);
}
/*
************************************************************
*              RESUME A SUSPENDED TASK BY PRIO             *
************************************************************
*/
U08 OS_TaskResume(U08 prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_TCB      *ptcb;

    if (prio >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if ((ptcb = OSTCBPrioTbl[prio]) == (OS_TCB *)NULL) {
        OS_EXIT_CRITICAL();
        return (OS_TASK_NOT_EXIST);
    }
#if OS_MUX_EN
    if (OSTCBPrioTbl[prio] == (OS_TCB *)1) {
        OS_EXIT_CRITICAL();
        return (OS_MUX_USED);
    }
#endif
    if (ptcb->OSTCBStat & OS_STAT_SUSP) {
        if ((ptcb->OSTCBStat &= ~(OS_STAT_SUSP)) == OS_STAT_RDY) {
            OSRdyGrp            |= OSMapTbl[prio >> 3];
            OSRdyTbl[prio >>3]  |= OSMapTbl[prio & 0x07];
            OS_EXIT_CRITICAL();
            OSSched();
        } else
            OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TASK_NOT_SUSP);
}
/*
************************************************************
*               RESUME A SUSPENDED TASK BY ID              *
************************************************************
*/
U08 OS_TaskIdResume(U08 id) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          prio;

    if (id >= OS_MAX_TASKS)
        return (OS_TASK_NOT_EXIST);
    OS_ENTER_CRITICAL();
    prio = OS_TaskGetPrio(id);
    if(prio != 0xFF) {
        prio = OS_TaskResume(prio);
        OS_EXIT_CRITICAL();
        return (prio);
    }
    OS_EXIT_CRITICAL();
    return (OS_TASK_NOT_EXIST);
}
#endif /* OS_TASK_EXT_EN */

/************************* END ****************************/


